@@ -42,8 +42,6 @@
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkcheckbutton.h>
 
-#include <nautilus-burn-drive.h>
-
 #include "burn-basics.h"
 #include "burn-session.h"
 #include "burn.h"
@@ -70,7 +68,7 @@ typedef struct BraseroBlankDialogPrivate BraseroBlankDialogPrivate;
 
 #define BRASERO_BLANK_DIALOG_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_BLANK_DIALOG, BraseroBlankDialogPrivate))
 
-static GObjectClass *parent_class = NULL;
+static BraseroToolDialogClass *parent_class = NULL;
 
 static guint
 brasero_blank_dialog_set_button (BraseroBurnSession *session,
@@ -172,17 +170,24 @@ brasero_blank_dialog_fast_toggled (GtkToggleButton *toggle,
 
 static void
 brasero_blank_dialog_drive_changed (BraseroToolDialog *dialog,
-				    NautilusBurnDrive *drive)
+				    BraseroMedium *medium)
 {
 	BraseroBlankDialogPrivate *priv;
+	BraseroDrive *drive;
 
 	priv = BRASERO_BLANK_DIALOG_PRIVATE (dialog);
+
+	if (medium)
+		drive = brasero_medium_get_drive (medium);
+	else
+		drive = NULL;
+
 	brasero_burn_session_set_burner (priv->session, drive);
 }
 
 static gboolean
 brasero_blank_dialog_activate (BraseroToolDialog *dialog,
-			       NautilusBurnDrive *drive)
+			       BraseroMedium *medium)
 {
 	BraseroBlankDialogPrivate *priv;
 	BraseroBlankDialog *self;
@@ -211,7 +216,7 @@ brasero_blank_dialog_activate (BraseroToolDialog *dialog,
 						   GTK_BUTTONS_CLOSE,
 						   _("Error Blanking:"));
 
-		gtk_window_set_title (GTK_WINDOW (self), _("Blanking finished"));
+		gtk_window_set_title (GTK_WINDOW (self), _("Blanking Finished"));
 
 		button = brasero_utils_make_button (_("Blank _Again"),
 						    NULL,
@@ -252,7 +257,7 @@ brasero_blank_dialog_activate (BraseroToolDialog *dialog,
 						  GTK_BUTTONS_NONE,
 						  _("The disc was successfully blanked:"));
 
-		gtk_window_set_title (GTK_WINDOW (self), _("Blanking finished"));
+		gtk_window_set_title (GTK_WINDOW (self), _("Blanking Finished"));
 
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 							  _("the disc is ready for use."));
@@ -348,7 +353,8 @@ brasero_blank_dialog_init (BraseroBlankDialog *obj)
 {
 	BraseroBlankDialogPrivate *priv;
 	BraseroPluginManager *manager;
-	NautilusBurnDrive *drive;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
 
 	priv = BRASERO_BLANK_DIALOG_PRIVATE (obj);
 
@@ -357,14 +363,17 @@ brasero_blank_dialog_init (BraseroBlankDialog *obj)
 					NULL,
 					"media-optical-blank");
 
-	drive = brasero_tool_dialog_get_drive (BRASERO_TOOL_DIALOG (obj));
+	medium = brasero_tool_dialog_get_medium (BRASERO_TOOL_DIALOG (obj));
+	drive = brasero_medium_get_drive (medium);
 
 	priv->session = brasero_burn_session_new ();
 	brasero_burn_session_set_flags (priv->session,
 				        BRASERO_BURN_FLAG_EJECT|
 				        BRASERO_BURN_FLAG_NOGRACE);
 	brasero_burn_session_set_burner (priv->session, drive);
-	nautilus_burn_drive_unref (drive);
+
+	if (medium)
+		g_object_unref (medium);
 
 	priv->output_sig = g_signal_connect (priv->session,
 					     "output-changed",
@@ -402,7 +411,7 @@ brasero_blank_dialog_new ()
 	BraseroBlankDialog *obj;
 
 	obj = BRASERO_BLANK_DIALOG (g_object_new (BRASERO_TYPE_BLANK_DIALOG,
-						  "title", _("Disc blanking"),
+						  "title", _("Disc Blanking"),
 						  NULL));
 	return GTK_WIDGET (obj);
 }
