@@ -42,12 +42,13 @@
 #include "brasero-utils.h"
 #include "burn-basics.h"
 #include "burn-medium.h"
+#include "burn-volume-obj.h"
 #include "burn-session.h"
 #include "burn-caps.h"
 #include "burn-plugin-manager.h"
 #include "brasero-disc-option-dialog.h"
 #include "brasero-dest-selection.h"
-#include "brasero-ncb.h"
+#include "burn-drive.h"
 #include "brasero-disc.h"
 
 G_DEFINE_TYPE (BraseroDiscOptionDialog, brasero_disc_option_dialog, GTK_TYPE_DIALOG);
@@ -61,7 +62,6 @@ struct _BraseroDiscOptionDialogPrivate {
 	guint caps_sig;
 	guint output_sig;
 
-	GtkWidget *video_toggle;
 	GtkWidget *joliet_toggle;
 	GtkWidget *multi_toggle;
 
@@ -95,6 +95,9 @@ brasero_disc_option_dialog_save_multi_state (BraseroDiscOptionDialog *dialog)
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
 	key = brasero_burn_session_get_config_key (priv->session, "multi");
+	if (!key)
+		return;
+
 	multi_on = (brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MULTI) != 0;
 
 	client = gconf_client_get_default ();
@@ -114,8 +117,10 @@ brasero_disc_option_dialog_load_multi_state (BraseroDiscOptionDialog *dialog)
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
 	/* That's only provided multi is not compulsory or unsupported */
-
 	key = brasero_burn_session_get_config_key (priv->session, "multi");
+	if (!key)
+		return;
+
 	client = gconf_client_get_default ();
 	multi_on = gconf_client_get_int (client, key, NULL);
 	g_object_unref (client);
@@ -129,54 +134,93 @@ brasero_disc_option_dialog_load_multi_state (BraseroDiscOptionDialog *dialog)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->multi_toggle), FALSE);
 }
 
+static gchar *
+brasero_disc_option_dialog_truncate_label (const gchar *label)
+{
+	const gchar *delim;
+	gchar *next_char;
+
+	/* find last possible character. We can't just do a tmp + 32 
+	 * since we don't know if we are at the start of a character */
+	delim = label;
+	while ((next_char = g_utf8_find_next_char (delim, NULL))) {
+		if (next_char - label > 32)
+			break;
+
+		delim = next_char;
+	}
+
+	return g_strndup (label, delim - label);
+}
+
 static gchar *
 brasero_disc_option_dialog_get_default_label (BraseroDiscOptionDialog *dialog)
 {
 	time_t t;
 	gchar buffer [128];
+	BraseroDrive *drive;
 	gchar *title_str = NULL;
 	BraseroTrackType source;
-	BraseroMedia media;
-	NautilusBurnDrive *drive;
 	BraseroDiscOptionDialogPrivate *priv;
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
 	brasero_burn_session_get_input_type (priv->session, &source);
-
 	drive = brasero_drive_selection_get_drive (BRASERO_DRIVE_SELECTION (priv->selection));
-	media = NCB_MEDIA_GET_STATUS (drive);
 
 	t = time (NULL);
 	strftime (buffer, sizeof (buffer), "%d %b %y", localtime (&t));
 
 	if (source.type == BRASERO_TRACK_TYPE_DATA) {
-		if ((media & BRASERO_MEDIUM_APPENDABLE)
-		&&  !brasero_burn_session_is_dest_file (priv->session))
-			title_str = nautilus_burn_drive_get_media_label (drive);
+		BraseroBurnFlag flags;
 
-		if (!title_str || title_str [0] == '\0')
+		flags = brasero_burn_session_get_flags (priv->session);
+		if (flags & BRASERO_BURN_FLAG_MERGE) {
+			BraseroMedium *medium;
+
+			medium = brasero_drive_get_medium (drive);
+			title_str = brasero_volume_get_name (BRASERO_VOLUME (medium));
+		}
+
+		if (!title_str || title_str [0] == '\0') {
+			/* NOTE to translators: the final string must not be over
+			 * 32 _bytes_ otherwise it gets truncated. */
 			title_str = g_strdup_printf (_("Data disc (%s)"), buffer);
+
+			if (strlen (title_str) > 32) {
+				g_free (title_str);
+				strftime (buffer, sizeof (buffer), "%F", localtime (&t));
+				title_str = g_strdup_printf ("Data disc %s", buffer);
+			}
+		}
 	}
-	else if (source.type == BRASERO_TRACK_TYPE_AUDIO)
+	else if (source.type == BRASERO_TRACK_TYPE_AUDIO) {
+		/* NOTE to translators: the final string must not be over
+		 * 32 _bytes_ */
 		title_str = g_strdup_printf (_("Audio disc (%s)"), buffer);
 
-	nautilus_burn_drive_unref (drive);
+		if (strlen (title_str) > 32) {
+			g_free (title_str);
+			strftime (buffer, sizeof (buffer), "%F", localtime (&t));
+			title_str = g_strdup_printf ("Audio disc %s", buffer);
+		}
+	}
+
+	if (drive)
+		g_object_unref (drive);
+
+	if (strlen (title_str) > 32) {
+		gchar *tmp;
+
+		tmp = brasero_disc_option_dialog_truncate_label (title_str);
+		g_free (title_str);
+
+		title_str = tmp;
+	}
+
 	return title_str;
 }
 
-static void
-brasero_disc_option_dialog_set_label (BraseroDiscOptionDialog *dialog)
-{
-	const gchar *label;
-	BraseroDiscOptionDialogPrivate *priv;
-
-	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
-
-	label = gtk_entry_get_text (GTK_ENTRY (priv->label));
-	brasero_burn_session_set_label (priv->session, label);
-}
-
 static gboolean
 brasero_disc_option_dialog_update_label (BraseroDiscOptionDialog *dialog)
 {
@@ -192,7 +236,7 @@ brasero_disc_option_dialog_update_label (BraseroDiscOptionDialog *dialog)
 	gtk_entry_set_text (GTK_ENTRY (priv->label), label);
 	g_free (label);
 
-	brasero_disc_option_dialog_set_label (dialog);
+	brasero_burn_session_set_label (priv->session, label);
 	return TRUE;
 }
 
@@ -250,70 +294,10 @@ brasero_disc_option_dialog_update_joliet (BraseroDiscOptionDialog *dialog)
 	return TRUE;
 }
 
-static gboolean
-brasero_disc_option_dialog_update_video (BraseroDiscOptionDialog *dialog)
-{
-	BraseroDiscOptionDialogPrivate *priv;
-	BraseroBurnResult result;
-	BraseroTrackType source;
-
-	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
-
-	if (!priv->video_toggle)
-		return FALSE;
-
-	/* the library must have the proper support and multi mustn't be on */
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->multi_toggle)))
-		goto turn_off;
-
-	brasero_burn_session_get_input_type (priv->session, &source);
-	source.subtype.fs_type |= BRASERO_IMAGE_FS_VIDEO;
-	result = brasero_burn_caps_is_input_supported (priv->caps,
-						       priv->session,
-						       &source);
-
-	if (result != BRASERO_BURN_OK)
-		goto turn_off;
-
-	if (GTK_WIDGET_IS_SENSITIVE (priv->video_toggle))
-		return FALSE;
-
-	gtk_widget_set_sensitive (priv->video_toggle, TRUE);
-
-	if (!priv->video_saved)
-		return TRUE;
-
-	brasero_burn_session_set_input_type (priv->session, &source);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->video_toggle), TRUE);
-
-	/* multi and video shouldn't be on at the same time
-	 * NOTE: in this case multi_toggle cannot be on since we checked
-	 * its state earlier on */
-	priv->multi_saved = FALSE;
-	gtk_widget_set_sensitive (priv->multi_toggle, FALSE);
-
-	return TRUE;
-
-turn_off:
-
-	if (!GTK_WIDGET_IS_SENSITIVE (priv->video_toggle))
-		return FALSE;
-
-	priv->video_saved = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->video_toggle));
-	if (priv->video_saved) {
-		source.subtype.fs_type &= ~BRASERO_IMAGE_FS_VIDEO;
-		brasero_burn_session_set_input_type (priv->session, &source);
-
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->video_toggle), FALSE);
-	}
-
-	gtk_widget_set_sensitive (priv->video_toggle, FALSE);
-	return TRUE;
-}
-
 static void
 brasero_disc_option_dialog_update_multi (BraseroDiscOptionDialog *dialog)
 {
+	BraseroTrackType input;
 	BraseroDiscOptionDialogPrivate *priv;
 	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
@@ -323,6 +307,14 @@ brasero_disc_option_dialog_update_multi (BraseroDiscOptionDialog *dialog)
 	if (!priv->multi_toggle)
 		return;
 
+	brasero_burn_session_get_input_type (priv->session, &input);
+	if (input.type == BRASERO_TRACK_TYPE_DATA
+	&& (input.subtype.fs_type & BRASERO_IMAGE_FS_VIDEO)
+	&& (brasero_burn_session_get_dest_media (priv->session) & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))) {
+		gtk_widget_set_sensitive (priv->multi_toggle, FALSE);
+		goto end;
+	}
+
 	/* Wipe out some flags before trying to see if MULTI is supported:
 	 * DAO/BLANK_BEFORE_WRITE don't really get along well with MULTI */
 	brasero_burn_session_remove_flag (priv->session,
@@ -354,20 +346,6 @@ brasero_disc_option_dialog_update_multi (BraseroDiscOptionDialog *dialog)
 		goto end;
 	}
 
-	/* to improve video DVD compatibility we don't allow to leave a disc
-	 * open and have a video DVD created */
-	if (priv->video_toggle
-	&&  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->video_toggle))) {
-		if (GTK_WIDGET_IS_SENSITIVE (priv->multi_toggle))
-			priv->multi_saved = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->multi_toggle));
-
-		brasero_burn_session_remove_flag (priv->session, BRASERO_BURN_FLAG_MULTI);
-
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->multi_toggle), FALSE);
-		gtk_widget_set_sensitive (priv->multi_toggle, FALSE);
-		goto end;
-	}
-
 	/* only load preferences if it is supported and not compulsory */
 	gtk_widget_set_sensitive (priv->multi_toggle, TRUE);
 	brasero_disc_option_dialog_load_multi_state (dialog);
@@ -413,8 +391,6 @@ brasero_disc_option_dialog_caps_changed (BraseroPluginManager *manager,
 	brasero_disc_option_dialog_update_multi (dialog);
 	/* update the joliet button */
 	brasero_disc_option_dialog_update_joliet (dialog);
-	/* update the video button */
-	brasero_disc_option_dialog_update_video (dialog);
 }
 
 static void
@@ -430,8 +406,6 @@ brasero_disc_option_dialog_output_changed (BraseroBurnSession *session,
 	brasero_disc_option_dialog_update_multi (dialog);
 	/* update the joliet button */
 	brasero_disc_option_dialog_update_joliet (dialog);
-	/* update the video button */
-	brasero_disc_option_dialog_update_video (dialog);
 
 	/* see if we need to update the label */
 	if (!priv->label_modified)
@@ -440,39 +414,9 @@ brasero_disc_option_dialog_output_changed (BraseroBurnSession *session,
 
 /**
  * These functions are used to update the session according to the states
- * of the buttons and entry 
+ * of the buttons and entry
  */
 
-static void
-brasero_disc_option_dialog_set_video (BraseroDiscOptionDialog *dialog)
-{
-	BraseroDiscOptionDialogPrivate *priv;
-	BraseroTrackType source;
-
-	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
-
-	if (!priv->video_toggle)
-		return;
-
-	/* NOTE: we don't check for the sensitive property since when
-	 * something is compulsory the button is active but insensitive
-	 */
-	brasero_burn_session_get_input_type (priv->session, &source);
-	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->video_toggle))) {
-		source.subtype.fs_type &= ~BRASERO_IMAGE_FS_VIDEO;
-
-		brasero_disc_option_dialog_update_multi (dialog);
-	}
-	else {
-		source.subtype.fs_type |= BRASERO_IMAGE_FS_VIDEO;
-
-		priv->multi_saved = FALSE;
-		gtk_widget_set_sensitive (priv->multi_toggle, FALSE);
-	}
-
-	brasero_burn_session_set_input_type (priv->session, &source);
-}
-
 static void
 brasero_disc_option_dialog_set_joliet (BraseroDiscOptionDialog *dialog)
 {
@@ -506,24 +450,12 @@ brasero_disc_option_dialog_set_multi (BraseroDiscOptionDialog *dialog)
 
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->multi_toggle))) {
 		brasero_burn_session_remove_flag (priv->session, BRASERO_BURN_FLAG_MULTI);
-		brasero_disc_option_dialog_update_video (dialog);
 		brasero_disc_option_dialog_save_multi_state (dialog);
 		return;
 	}
 
 	brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_MULTI);
 	brasero_disc_option_dialog_save_multi_state (dialog);
-
-	if (!priv->video_toggle)
-		return;
-
-	/* to improve video DVD compatibility we don't allow to leave a disc
-	 * open and have a video DVD created.
-	 * NOTE: video and multi buttons are antithetic so if the user pressed
-	 * this one that means that video wasn't active so no need to set video
-	 * to FALSE*/
-	priv->video_saved = FALSE;
-	gtk_widget_set_sensitive (priv->video_toggle, FALSE);
 }
 
 static void
@@ -533,13 +465,6 @@ brasero_disc_option_dialog_multi_toggled (GtkToggleButton *multi_toggle,
 	brasero_disc_option_dialog_set_multi (dialog);
 }
 
-static void
-brasero_disc_option_dialog_video_toggled (GtkToggleButton *video_toggle,
-					  BraseroDiscOptionDialog *dialog)
-{
-	brasero_disc_option_dialog_set_video (dialog);
-}
-
 static void
 brasero_disc_option_dialog_joliet_toggled_cb (GtkToggleButton *toggle,
 					      BraseroDiscOptionDialog *dialog)
@@ -551,39 +476,129 @@ brasero_disc_option_dialog_joliet_toggled_cb (GtkToggleButton *toggle,
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
-	brasero_disc_option_dialog_set_joliet (dialog);
 	if (!GTK_WIDGET_VISIBLE (dialog)) {
 		gtk_widget_show (GTK_WIDGET (dialog));
 		hide = TRUE;
 	}
 
-	if (priv->joliet_warning)
+	if (priv->joliet_warning) {
+		brasero_disc_option_dialog_set_joliet (dialog);
 		return;
+	}
+
+	priv->joliet_warning = TRUE;
 
 	message = gtk_message_dialog_new (GTK_WINDOW (dialog),
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_INFO,
-					  GTK_BUTTONS_CLOSE,
-					  _("Some files don't have a suitable name for a Windows-compatible CD:"));
+					  GTK_BUTTONS_NONE,
+					  _("Should files be renamed to be windows-compatible?"));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("their names will be changed and truncated to 64 characters."));
+						  _("Some files don't have a suitable name for a Windows-compatible CD. Those names will be changed and truncated to 64 characters."));
+
+	gtk_window_set_title (GTK_WINDOW (message), _("Windows Compatibility"));
+
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Don't rename"),
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Rename"),
+			       GTK_RESPONSE_YES);
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Windows compatibility"));
 	answer = gtk_dialog_run (GTK_DIALOG (message));
 	gtk_widget_destroy (message);
+
+	if (answer != GTK_RESPONSE_YES)
+		gtk_toggle_button_set_active (toggle, FALSE);
+	else
+		brasero_disc_option_dialog_set_joliet (dialog);
+
+	priv->joliet_warning = FALSE;
+}
+
+static void
+brasero_disc_option_label_insert_text (GtkEditable *editable,
+				       const gchar *text,
+				       gint length,
+				       gint *position,
+				       gpointer data)
+{
+	BraseroDiscOptionDialogPrivate *priv;
+	const gchar *label;
+	gchar *new_text;
+	gint new_length;
+	gchar *current;
+	gint max_len;
+	gchar *prev;
+	gchar *next;
+
+	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (data);	
+
+	/* check if this new text will fit in 32 _bytes_ long buffer */
+	label = gtk_entry_get_text (GTK_ENTRY (priv->label));
+	max_len = 32 - strlen (label) - length;
+	if (max_len >= 0)
+		return;
+
+	gdk_beep ();
+
+	/* get the last character '\0' of the text to be inserted */
+	new_length = length;
+	new_text = g_strdup (text);
+	current = g_utf8_offset_to_pointer (new_text, g_utf8_strlen (new_text, -1));
+
+	/* don't just remove one character in case there was many more
+	 * that were inserted at the same time through DND, paste, ... */
+	prev = g_utf8_find_prev_char (new_text, current);
+	if (!prev) {
+		/* no more characters so no insertion */
+		g_signal_stop_emission_by_name (editable, "insert_text"); 
+		g_free (new_text);
+		return;
+	}
+
+	do {
+		next = current;
+		current = prev;
+
+		prev = g_utf8_find_prev_char (new_text, current);
+		if (!prev) {
+			/* no more characters so no insertion */
+			g_signal_stop_emission_by_name (editable, "insert_text"); 
+			g_free (new_text);
+			return;
+		}
+
+		new_length -= next - current;
+		max_len += next - current;
+	} while (max_len < 0 && new_length > 0);
+
+	*current = '\0';
+	g_signal_handlers_block_by_func (editable,
+					 (gpointer) brasero_disc_option_label_insert_text,
+					 data);
+	gtk_editable_insert_text (editable, new_text, new_length, position);
+	g_signal_handlers_unblock_by_func (editable,
+					   (gpointer) brasero_disc_option_label_insert_text,
+					   data);
+
+	g_signal_stop_emission_by_name (editable, "insert_text");
+	g_free (new_text);
 }
 
 static void
 brasero_disc_option_label_changed (GtkEditable *editable,
 				   BraseroDiscOptionDialog *dialog)
 {
+	const gchar *label;
 	BraseroDiscOptionDialogPrivate *priv;
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
-	brasero_disc_option_dialog_set_label (dialog);
+	label = gtk_entry_get_text (GTK_ENTRY (priv->label));
+	brasero_burn_session_set_label (priv->session, label);
 	priv->label_modified = 1;
 }
 
@@ -591,19 +606,21 @@ static void
 brasero_disc_option_dialog_title_widget (BraseroDiscOptionDialog *dialog)
 {
 	BraseroDiscOptionDialogPrivate *priv;
-	BraseroTrackType type;
 	gchar *title_str = NULL;
+	BraseroTrackType type;
 	gchar *label = NULL;
 	GtkWidget *widget;
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
-	if (!priv->label) {
+	if (!priv->label)
 		priv->label = gtk_entry_new ();
-		gtk_entry_set_max_length (GTK_ENTRY (priv->label), 32);
-	}
 
 	priv->label_modified = 0;
+	g_signal_connect (priv->label,
+			  "insert_text",
+			  G_CALLBACK (brasero_disc_option_label_insert_text),
+			  dialog);
 	g_signal_connect (priv->label,
 			  "changed",
 			  G_CALLBACK (brasero_disc_option_label_changed),
@@ -611,10 +628,9 @@ brasero_disc_option_dialog_title_widget (BraseroDiscOptionDialog *dialog)
 
 	title_str = brasero_disc_option_dialog_get_default_label (dialog);
 	gtk_entry_set_text (GTK_ENTRY (priv->label), title_str);
+	brasero_burn_session_set_label (priv->session, title_str);
 	g_free (title_str);
 
-	brasero_disc_option_dialog_set_label (dialog);
-
 	brasero_burn_session_get_input_type (priv->session, &type);
 	if (type.type == BRASERO_TRACK_TYPE_DATA)
 		label = g_strdup (_("<b>Label of the disc</b>"));
@@ -632,35 +648,6 @@ brasero_disc_option_dialog_title_widget (BraseroDiscOptionDialog *dialog)
 			    0);
 }
 
-static gboolean
-brasero_disc_option_dialog_video_widget (BraseroDiscOptionDialog *dialog)
-{
-	BraseroDiscOptionDialogPrivate *priv;
-	BraseroTrackType type;
-
-	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
-
-	/* Two conditions to create this widget:
-	 * the track must have the necessary files
-	 * the library must have the proper plugin */
-	brasero_burn_session_get_input_type (priv->session, &type);
-	if (!(type.type & BRASERO_IMAGE_FS_VIDEO))
-		return FALSE;
-
-	priv->video_toggle = gtk_check_button_new_with_mnemonic (_("Create a vid_eo DVD"));
-	g_signal_connect (priv->video_toggle,
-			  "toggled",
-			  G_CALLBACK (brasero_disc_option_dialog_video_toggled),
-			  dialog);
-	gtk_widget_set_tooltip_text (priv->video_toggle,
-			      _("Create a video DVD that can be played by all DVD readers"));
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->video_toggle), TRUE);
-	brasero_disc_option_dialog_update_video (dialog);
-
-	return TRUE;
-}
-
 static gboolean
 brasero_disc_option_dialog_joliet_widget (BraseroDiscOptionDialog *dialog)
 {
@@ -698,6 +685,7 @@ brasero_disc_option_dialog_multi_widget (BraseroDiscOptionDialog *dialog)
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 	priv->multi_toggle = gtk_check_button_new_with_mnemonic (_("Leave the disc _open to add other files later"));
+
 	g_signal_connect (priv->multi_toggle,
 			  "toggled",
 			  G_CALLBACK (brasero_disc_option_dialog_multi_toggled),
@@ -714,12 +702,10 @@ brasero_disc_option_dialog_add_data_options (BraseroDiscOptionDialog *dialog)
 {
 	BraseroDiscOptionDialogPrivate *priv;
 	GtkWidget *widget = NULL;
-	BraseroTrackType source;
 	GtkWidget *options;
 
 	priv = BRASERO_DISC_OPTION_DIALOG_PRIVATE (dialog);
 
-	brasero_burn_session_get_input_type (priv->session, &source);
 
 	/* create the options */
 	widget = gtk_vbox_new (FALSE, 0);
@@ -735,13 +721,9 @@ brasero_disc_option_dialog_add_data_options (BraseroDiscOptionDialog *dialog)
 	/* general options */
 	brasero_disc_option_dialog_joliet_widget (dialog);
 
-	/* video toggle */
-	brasero_disc_option_dialog_video_widget (dialog);
-
 	options = brasero_utils_pack_properties (_("<b>Disc options</b>"),
 						 priv->multi_toggle,
 						 priv->joliet_toggle,
-						 priv->video_toggle,
 						 NULL);
 	gtk_box_pack_start (GTK_BOX (widget), options, FALSE, FALSE, 0);
 
@@ -820,11 +802,14 @@ brasero_disc_option_dialog_set_disc (BraseroDiscOptionDialog *dialog,
 
 	brasero_burn_session_get_input_type (priv->session, &type);
 	if (type.type == BRASERO_TRACK_TYPE_DATA) {
-		brasero_drive_selection_show_file_drive (BRASERO_DRIVE_SELECTION (priv->selection), TRUE);
+	brasero_drive_selection_set_type_shown (BRASERO_DRIVE_SELECTION (priv->selection),
+						BRASERO_MEDIA_TYPE_WRITABLE|
+						BRASERO_MEDIA_TYPE_FILE);
 		brasero_disc_option_dialog_add_data_options (dialog);
 	}
 	else if (type.type == BRASERO_TRACK_TYPE_AUDIO) {
-		brasero_drive_selection_show_file_drive (BRASERO_DRIVE_SELECTION (priv->selection), FALSE);
+		brasero_drive_selection_set_type_shown (BRASERO_DRIVE_SELECTION (priv->selection),
+							BRASERO_MEDIA_TYPE_WRITABLE);
 		brasero_disc_option_dialog_add_audio_options (dialog);
 	}
 }
@@ -874,7 +859,7 @@ brasero_disc_option_dialog_init (BraseroDiscOptionDialog *obj)
 	priv->button = brasero_utils_make_button (_("_Burn"),
 						  NULL,
 						  "media-optical-burn",
-						  GTK_ICON_SIZE_LARGE_TOOLBAR);
+						  GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (priv->button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (obj),
 				      priv->button,
@@ -902,13 +887,10 @@ brasero_disc_option_dialog_init (BraseroDiscOptionDialog *obj)
 			  G_CALLBACK (brasero_disc_option_dialog_valid_media_cb),
 			  obj);
 
-	brasero_drive_selection_select_default_drive (BRASERO_DRIVE_SELECTION (priv->selection),
-						      BRASERO_MEDIUM_WRITABLE);
-
-	options = brasero_utils_pack_properties (_("<b>Select a drive to write to</b>"),
+	options = brasero_utils_pack_properties (_("<b>Select a disc to write to</b>"),
 						 priv->selection,
 						 NULL);
-	gtk_widget_show_all (options);
+	gtk_widget_show (options);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox),
 			    options,
@@ -970,10 +952,10 @@ GtkWidget *
 brasero_disc_option_dialog_new ()
 {
 	BraseroDiscOptionDialog *obj;
-	
+
 	obj = BRASERO_DISC_OPTION_DIALOG (g_object_new (BRASERO_TYPE_DISC_OPTION_DIALOG,
-							"title", _("Disc burning setup"),
+							"title", _("Disc Burning Setup"),
 							NULL));
-	
+
 	return GTK_WIDGET (obj);
 }
