@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
  * brasero is free software.
  * 
@@ -30,6 +30,8 @@
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
+#include <gio/gio.h>
+
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkcombobox.h>
@@ -41,14 +43,10 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkbox.h>
 
-#include <nautilus-burn-drive.h>
-
-#include <libgnomevfs/gnome-vfs.h>
-
 #include "burn-basics.h"
 #include "burn-medium.h"
 #include "burn-debug.h"
-#include "brasero-ncb.h"
+#include "burn-drive.h"
 #include "brasero-utils.h"
 #include "brasero-drive-properties.h"
 
@@ -135,12 +133,12 @@ void
 brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
 				     const gchar *path)
 {
+	GFile *file;
 	gchar *string;
-	gchar *uri_str;
+	GFileInfo *info;
 	gchar *directory;
-	GnomeVFSURI *uri;
-	BraseroBurnResult result;
-	GnomeVFSFileSize vol_size = 0;
+	GError *error = NULL;
+	guint64 vol_size = 0;
 	BraseroDrivePropertiesPrivate *priv;
 
 	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (self);
@@ -154,26 +152,33 @@ brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
 
 	/* get the volume free space */
 	directory = g_path_get_dirname (path);
-	uri_str = gnome_vfs_get_uri_from_local_path (directory);
+	file = g_file_new_for_commandline_arg (directory);
 	g_free (directory);
 
-	uri = gnome_vfs_uri_new (uri_str);
-	g_free (uri_str);
-
-	if (uri == NULL) {
+	if (file == NULL) {
 		BRASERO_BURN_LOG ("impossible to retrieve size for %s", path);
 		gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), _("unknown"));
 		return;
 	}
 
-	result = gnome_vfs_get_volume_free_space (uri, &vol_size);
-	if (result != GNOME_VFS_OK) {
-		BRASERO_BURN_LOG ("impossible to retrieve size for %s", path);
+	info = g_file_query_filesystem_info (file,
+					     G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+					     NULL,
+					     &error);
+	g_object_unref (file);
+
+	if (error) {
+		g_object_unref (info);
+
+		BRASERO_BURN_LOG ("impossible to retrieve size for %s (%s)", path, error->message);
+		g_error_free (error);
+
 		gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), _("unknown"));
 		return;
 	}
 
-	gnome_vfs_uri_unref (uri);
+	vol_size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+	g_object_unref (info);
 
 	string = brasero_utils_get_size_string (vol_size, TRUE, TRUE);
 	gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), string);
@@ -240,14 +245,14 @@ brasero_drive_properties_format_disc_speed (BraseroMedia media,
 {
 	gchar *text;
 
-	if (media & BRASERO_MEDIUM_DVD)
-		text = g_strdup_printf (_("%d x (DVD)"),
+	if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+		text = g_strdup_printf (_("%.1f x (DVD)"),
 					BRASERO_RATE_TO_SPEED_DVD (rate));
 	else if (media & BRASERO_MEDIUM_CD)
-		text = g_strdup_printf (_("%d x (CD)"),
+		text = g_strdup_printf (_("%.1f x (CD)"),
 					BRASERO_RATE_TO_SPEED_CD (rate));
 	else
-		text = g_strdup_printf (_("%d x (DVD) %d x (CD)"),
+		text = g_strdup_printf (_("%.1f x (DVD) %.1f x (CD)"),
 					BRASERO_RATE_TO_SPEED_DVD (rate),
 					BRASERO_RATE_TO_SPEED_CD (rate));
 
@@ -256,25 +261,25 @@ brasero_drive_properties_format_disc_speed (BraseroMedia media,
 
 void
 brasero_drive_properties_set_drive (BraseroDriveProperties *self,
-				    NautilusBurnDrive *drive,
+				    BraseroDrive *drive,
 				    gint64 default_rate)
 {
 	BraseroDrivePropertiesPrivate *priv;
+	BraseroMedium *medium;
 	BraseroMedia media;
 	GtkTreeModel *model;
 	gchar *display_name;
 	GtkTreeIter iter;
-	gint64 max_rate;
-	gchar *header;
-	gint64 rate;
-	gint64 step;
 	gchar *max_text;
+	gint64 *rates;
+	gchar *header;
 	gchar *text;
+	guint i;
 
 	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (self);
 
 	/* set the header of the dialog */
-	display_name = nautilus_burn_drive_get_name_for_display (drive);
+	display_name = brasero_drive_get_display_name (drive);
 	header = g_strdup_printf (_("Properties of %s"), display_name);
 	g_free (display_name);
 
@@ -282,13 +287,9 @@ brasero_drive_properties_set_drive (BraseroDriveProperties *self,
 	g_free (header);
 
 	/* Speed combo */
-	media = NCB_MEDIA_GET_STATUS (drive);
-	max_rate = NCB_MEDIA_GET_MAX_WRITE_RATE (drive);
-	if (media & BRASERO_MEDIUM_CD)
-		step = CD_RATE;
-	else
-		step = DVD_RATE;
-
+	medium = brasero_drive_get_medium (drive);
+	media = brasero_medium_get_status (medium);
+	rates = brasero_medium_get_write_speeds (medium);
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->speed));
 
 	max_text = g_strdup_printf (_("Max speed"));
@@ -296,26 +297,42 @@ brasero_drive_properties_set_drive (BraseroDriveProperties *self,
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    PROP_TEXT, max_text,
-			    PROP_RATE, max_rate,
+			    PROP_RATE, rates [0],
 			    -1);
 	g_free (max_text);
 
-	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->speed), 0);
-
-	for (rate = max_rate - step; rate > step; rate -= step * 2) {
-		GtkTreeIter iter;
-
-		text = brasero_drive_properties_format_disc_speed (media, rate);
+	/* fill model */
+	for (i = 0; rates [i] != 0; i ++) {
+		text = brasero_drive_properties_format_disc_speed (media, rates [i]);
 		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 				    PROP_TEXT, text,
-				    PROP_RATE, rate,
+				    PROP_RATE, rates [i],
 				    -1);
 		g_free (text);
+	}
+
+	/* Set active one preferably max speed */
+	gtk_tree_model_get_iter_first (model, &iter);
+	do {
+		gint64 rate;
+
+		gtk_tree_model_get (model, &iter,
+				    PROP_RATE, &rate,
+				    -1);
 
 		/* we do this to round things and get the closest possible speed */
-		if ((rate / step) == (default_rate / step))
+		if ((rate / 1024) == (default_rate / 1024)) {
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->speed), &iter);
+			break;
+		}
+
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	/* make sure at least one is active */
+	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->speed), &iter)) {
+		gtk_tree_model_get_iter_first (model, &iter);
+		gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->speed), &iter);
 	}
 }
 
