@@ -37,8 +37,6 @@
 
 #include <gconf/gconf-client.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #include <gst/gst.h>
 #include <gst/interfaces/xoverlay.h>
 
@@ -517,9 +515,9 @@ brasero_player_bacon_set_volume (BraseroPlayerBacon *bacon, gdouble volume)
 	if (!bacon->priv->pipe)
 		return;
 
-	volume = CLAMP (volume, 0.0, 5.0);
+	volume = CLAMP (volume, 0, 100) / 100.0;
 	g_object_set (bacon->priv->pipe,
-		      "volume", volume * 5.0,
+		      "volume", volume * 100,
 		      NULL);
 }
 
@@ -545,6 +543,7 @@ brasero_player_bacon_bus_messages (GstBus *bus,
 {
 	BraseroPlayerBaconState value;
 	GstStateChangeReturn result;
+	GError *error = NULL;
 	GstState state;
 
 	switch (GST_MESSAGE_TYPE (msg)) {
@@ -555,6 +554,9 @@ brasero_player_bacon_bus_messages (GstBus *bus,
 		break;
 
 	case GST_MESSAGE_ERROR:
+		gst_message_parse_error (msg, &error, NULL);
+		BRASERO_BURN_LOG ("%s", error->message);
+
 		g_signal_emit (bacon,
 			       brasero_player_bacon_signals [STATE_CHANGED_SIGNAL],
 			       0,
@@ -634,9 +636,6 @@ brasero_player_bacon_set_pos (BraseroPlayerBacon *bacon,
 				    &format,
 				    &duration);
 
-	if ((gint64) pos > duration || pos > bacon->priv->end)
-		return FALSE;
-
 	return gst_element_seek (bacon->priv->pipe,
 				 1.0,
 				 GST_FORMAT_TIME,
