@@ -36,13 +36,12 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkcontainer.h>
 
-#include <nautilus-burn-drive.h>
-#include <nautilus-burn-drive-monitor.h>
-
 #include "brasero-project-size.h"
-#include "brasero-ncb.h"
-#include "burn-volume.h"
-#include "utils.h"
+#include "burn-caps.h"
+#include "burn-volume-obj.h"
+#include "brasero-utils.h"
+#include "burn-medium-monitor.h"
+#include "burn-medium.h"
 
 static void brasero_project_size_class_init (BraseroProjectSizeClass *klass);
 static void brasero_project_size_init (BraseroProjectSize *sp);
@@ -83,18 +82,17 @@ brasero_project_size_forall_children (GtkContainer *container,
 				      GtkCallback callback,
 				      gpointer callback_data);
 
-struct _BraseroDrive {
+struct _BraseroDriveSize {
 	gint64 sectors;
 	gint64 free_space;
-	BraseroMediumInfo media;
-	NautilusBurnDrive *drive;
+	BraseroMedia media;
+	BraseroMedium *medium;
 };
-typedef struct _BraseroDrive BraseroDrive;
+typedef struct _BraseroDriveSize BraseroDriveSize;
 
 struct _BraseroProjectSizePrivate {
 	GtkWidget *menu;
-	GtkTooltips *tooltips;
-
+	
 	GtkWidget *frame;
 	GtkWidget *arrow;
 	GtkWidget *button;
@@ -107,7 +105,7 @@ struct _BraseroProjectSizePrivate {
 
 	gint64 sectors;
 	GList *drives;
-	BraseroDrive *current;
+	BraseroDriveSize *current;
 
 	guint is_audio_context:1;
 	guint was_chosen:1;
@@ -204,20 +202,20 @@ brasero_project_size_class_init (BraseroProjectSizeClass *klass)
 static void
 brasero_project_size_add_default_medias (BraseroProjectSize *self)
 {
-	const BraseroDrive drives [] =  { {333000, 333000, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK, NULL},
+	const BraseroDriveSize drives [] =  { {333000, 333000, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK, NULL},
 					  {360000, 360000, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK, NULL},
 					  {405000, 405000, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK, NULL},
 					  {450000, 450000, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK, NULL},
 					  {2295104, 2295104, BRASERO_MEDIUM_DVDR|BRASERO_MEDIUM_BLANK, NULL},
 					  {4150390, 4150390, BRASERO_MEDIUM_DVDR_DL|BRASERO_MEDIUM_BLANK, NULL},
 					  { 0 } };
-	const BraseroDrive *iter;
+	const BraseroDriveSize *iter;
 
 	for (iter = drives; iter->sectors; iter ++) {
-		BraseroDrive *drive;
+		BraseroDriveSize *drive;
 
-		drive = g_new0 (BraseroDrive, 1);
-	    	memcpy (drive, iter, sizeof (BraseroDrive));
+		drive = g_new0 (BraseroDriveSize, 1);
+	    	memcpy (drive, iter, sizeof (BraseroDriveSize));
 		self->priv->drives = g_list_prepend (self->priv->drives, drive);
 	}
 }
@@ -232,14 +230,10 @@ brasero_project_size_init (BraseroProjectSize *obj)
 	obj->priv = g_new0 (BraseroProjectSizePrivate, 1);
 	obj->priv->text_layout = gtk_widget_create_pango_layout (GTK_WIDGET (obj), "");
 
-	obj->priv->tooltips = gtk_tooltips_new ();
-
 	brasero_project_size_add_default_medias (obj);
 
 	obj->priv->button = gtk_toggle_button_new ();
-	gtk_tooltips_set_tip (obj->priv->tooltips,
-			      obj->priv->button,
-			      _("Show the available media to be burnt"),
+	gtk_widget_set_tooltip_text (obj->priv->button,
 			      _("Show the available media to be burnt"));
 	gtk_container_set_border_width (GTK_CONTAINER (obj->priv->button), 0);
 	g_signal_connect (obj->priv->button,
@@ -275,11 +269,6 @@ brasero_project_size_finalize (GObject *object)
 
 	cobj = BRASERO_PROJECT_SIZE (object);
 
-	if (cobj->priv->tooltips) {
-		g_object_ref_sink (GTK_OBJECT (cobj->priv->tooltips));
-		cobj->priv->tooltips = NULL;
-	}
-
 	if (cobj->priv->frame) {
 		gtk_widget_unparent (cobj->priv->frame);
 		cobj->priv->frame = NULL;
@@ -311,11 +300,12 @@ brasero_project_size_finalize (GObject *object)
 	}
 
 	for (iter = cobj->priv->drives; iter; iter = iter->next) {
-		BraseroDrive *drive;
+		BraseroDriveSize *drive;
 
 		drive = iter->data;
-		if (drive->drive)
-			nautilus_burn_drive_unref (drive->drive);
+
+		if (drive->medium)
+			g_object_unref (drive->medium);
 
 		g_free (drive);
 	}
@@ -413,7 +403,7 @@ brasero_project_size_get_ruler_min_width (BraseroProjectSize *self,
 	PangoRectangle extents = { 0 };
 	PangoLayout *layout;
 
-	BraseroDrive *drive;
+	BraseroDriveSize *drive;
 
 	gint max_width, max_height, total;
 	gint interval_size;
@@ -429,7 +419,7 @@ brasero_project_size_get_ruler_min_width (BraseroProjectSize *self,
 	/* the number of interval needs to be reasonable, not over 8 not under 5 */
 	if (self->priv->is_audio_context)
 		interval_size = AUDIO_INTERVAL_CD;
-	else if (self->priv->current->media & BRASERO_MEDIUM_DVD)
+	else if (self->priv->current->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
 		interval_size = DATA_INTERVAL_DVD;
 	else
 		interval_size = DATA_INTERVAL_CD;
@@ -446,8 +436,8 @@ brasero_project_size_get_ruler_min_width (BraseroProjectSize *self,
 	 * rewritable and multisession is off. That also applies to the widget
 	 * to select recorders.	 
 	 */
-	if (drive->drive
-	&& (NCB_MEDIA_GET_STATUS (drive->drive) & BRASERO_MEDIUM_REWRITABLE)
+	if (drive->medium
+	&& (brasero_medium_get_status (drive->medium) & BRASERO_MEDIUM_REWRITABLE)
 	&& !self->priv->multi)
 		total = drive->sectors > self->priv->sectors ? drive->sectors:self->priv->sectors;
 	else
@@ -494,7 +484,8 @@ brasero_project_size_get_media_string (BraseroProjectSize *self)
 {
 	gint64 disc_size;
 	gchar *text = NULL;
-	BraseroDrive *drive;
+	BraseroDriveSize *drive;
+	BraseroMedia status;
 	gchar *drive_name = NULL;
 	gchar *disc_sectors_str = NULL;
 	gchar *selection_size_str = NULL;
@@ -503,8 +494,9 @@ brasero_project_size_get_media_string (BraseroProjectSize *self)
 	if (!drive)
 		return NULL;
 
-	if (drive->drive
-	&& (NCB_MEDIA_GET_STATUS (drive->drive) & BRASERO_MEDIUM_REWRITABLE)
+	status = brasero_medium_get_status (drive->medium);
+	if (drive->medium
+	&& (status & BRASERO_MEDIUM_REWRITABLE)
 	&& !self->priv->multi)
 		disc_size = drive->sectors;
 	else
@@ -515,33 +507,57 @@ brasero_project_size_get_media_string (BraseroProjectSize *self)
 		/* this is an empty drive */
 		return NULL;
 	}
-	else if (drive->sectors == -1) {
+
+	if (status == BRASERO_MEDIUM_BUSY) {
 		gchar *name;
 
-		/* this is a mounted drive */
-		name = nautilus_burn_drive_get_name_for_display (drive->drive);
-		disc_sectors_str = g_strdup_printf (_("(mounted drive) <i>%s</i>"), name);
+		/* this is a busy drive */
+		name = brasero_volume_get_display_label (BRASERO_VOLUME (drive->medium), FALSE);
+		disc_sectors_str = g_strdup_printf (_("<i>%s</i> is busy"), name);
 		g_free (name);
+
+		return disc_sectors_str;
 	}
-	else {
-		disc_sectors_str = brasero_utils_get_sectors_string (disc_size,
-								     self->priv->is_audio_context,
-								     TRUE,
-								     TRUE);
 
-		if (drive->drive) {
-			/* we ellipsize to max characters to avoid having
-			 * a too long string with the drive full name. */
-			drive_name = nautilus_burn_drive_get_name_for_display (drive->drive);
-			if (strlen (drive_name) > 19) {
-				gchar *tmp;
+	if (drive->sectors == -1) {
+		gchar *name;
 
-				tmp = g_strdup_printf ("%.16s...", drive_name);
-				g_free (drive_name);
-				drive_name = tmp;
-			}
+		/* this is a drive probably not fully supported by brasero */
+		name = brasero_volume_get_display_label (BRASERO_VOLUME (drive->medium), FALSE);
+		disc_sectors_str = g_strdup_printf (_("<i>%s</i> not properly supported"), name);
+		g_free (name);
+
+		return disc_sectors_str;
+	}
+
+	if (status == BRASERO_MEDIUM_UNSUPPORTED) {
+		gchar *name;
+
+		/* this is an unsupported medium */
+		name = brasero_volume_get_display_label (BRASERO_VOLUME (drive->medium), FALSE);
+		disc_sectors_str = g_strdup_printf (_("The disc in <i>%s</i> is not supported"), name);
+		g_free (name);
+
+		return disc_sectors_str;
+	}
+
+	disc_sectors_str = brasero_utils_get_sectors_string (disc_size,
+							     self->priv->is_audio_context,
+							     TRUE,
+							     TRUE);
+
+	if (drive->medium) {
+		/* we ellipsize to max characters to avoid having
+		 * a too long string with the drive full name. */
+		drive_name = brasero_volume_get_display_label (BRASERO_VOLUME (drive->medium), TRUE);
+/*		if (strlen (drive_name) > 19) {
+			gchar *tmp;
+
+			tmp = g_strdup_printf ("%.16s...", drive_name);
+			g_free (drive_name);
+			drive_name = tmp;
 		}
-	}
+*/	}
 
 	selection_size_str = brasero_utils_get_sectors_string (self->priv->sectors,
 							       self->priv->is_audio_context,
@@ -561,17 +577,15 @@ brasero_project_size_get_media_string (BraseroProjectSize *self)
 	}
 	else if (self->priv->sectors == 0) {
 		if (drive_name)
-			text = g_strdup_printf (_("<b>Empty</b> (%s / %s in <i>%s</i>)"),
-						selection_size_str,
+			text = g_strdup_printf (_("<b>Empty</b> (%s free for <i>%s</i>)"),
 						disc_sectors_str,
 						drive_name);
 		else
-			text = g_strdup_printf (_("<b>Empty</b> (%s / %s)"),
-						selection_size_str,
+			text = g_strdup_printf (_("<b>Empty</b> (%s free)"),
 						disc_sectors_str);
 	}
 	else if (drive_name)
-		text = g_strdup_printf ("%s / %s (in <i>%s</i>)",
+		text = g_strdup_printf ("%s / %s (for <i>%s</i>)",
 					selection_size_str,
 					disc_sectors_str,
 					drive_name);
@@ -591,41 +605,30 @@ static void
 brasero_project_size_size_request (GtkWidget *widget,
 				   GtkRequisition *requisition)
 {
-	gint width, height, text_width, ruler_width, ruler_height;
+	gint width, height, ruler_width, ruler_height;
 	PangoRectangle extents = { 0 };
 	BraseroProjectSize *self;
 	GtkRequisition req;
+	gchar *text;
 
 	self = BRASERO_PROJECT_SIZE (widget);
 
-	if (self->priv->text_layout) {
-		gchar *text;
 
-		/* Set markup every time a size change this function is called */
-		text = brasero_project_size_get_media_string (self);
-		pango_layout_set_markup (self->priv->text_layout, text, -1);
-		g_free (text);
-
-		pango_layout_get_pixel_extents (self->priv->text_layout,
-						NULL,
-						&extents);
-		text_width = extents.width + BRASERO_PROJECT_SIZE_SPACE * 2;
-	}
-	else
-		text_width = 0;
+	/* Set markup every time a size change this function is called */
+	text = brasero_project_size_get_media_string (self);
+	pango_layout_set_markup (self->priv->text_layout, text, -1);
+	g_free (text);
 
 	brasero_project_size_get_ruler_min_width (self, &ruler_width, &ruler_height);
 	gtk_widget_size_request (self->priv->button, &req);
 
-	width = MAX (ruler_width, text_width) +
-		self->priv->frame->style->xthickness * 2 +
-		req.width;
+	width = self->priv->frame->style->xthickness * 2 + req.width * 2;
 
 	height = extents.height + self->priv->frame->style->ythickness * 2;
 	height = MAX (height, req.height);
 	height += ruler_height;
-	
-	requisition->height = height > BRASERO_PROJECT_SIZE_HEIGHT ? height:BRASERO_PROJECT_SIZE_HEIGHT;
+
+	requisition->height = height;
 	requisition->width = width;
 
 	self->priv->ruler_height = ruler_height;
@@ -641,9 +644,6 @@ brasero_project_size_size_allocate (GtkWidget *widget,
 
 	gboolean is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
 
-	if (!GTK_WIDGET_REALIZED (widget))
-		return;
-
 	widget->allocation = *allocation;
 	if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_NO_WINDOW (widget)) {
 		gdk_window_move_resize (widget->window,
@@ -660,18 +660,18 @@ brasero_project_size_size_allocate (GtkWidget *widget,
 	/* NOTE: since we've got our own window, we don't need to take into
 	 * account alloc.x and alloc.y */
 	if (is_rtl)
-		alloc.x = allocation->width - req.width;
+		alloc.x = allocation->width - req.width - 1;
 	else
 		alloc.x = - 1;
 
 	alloc.y = - 1;
 	alloc.width = MAX (1, req.width + self->priv->frame->style->xthickness * 2 - 2);
-	alloc.height = MAX (req.height + self->priv->frame->style->xthickness * 2 - 2, allocation->height - self->priv->ruler_height);
+	alloc.height = MAX (req.height - 2, allocation->height - self->priv->ruler_height);
 	gtk_widget_size_allocate (self->priv->button, &alloc);
 
 	/* allocate the size for the arrow we want to draw on the button */
 	if (is_rtl)
-		alloc.x = alloc.width / 4;
+		alloc.x += alloc.width / 2;
 	else
 		alloc.x = alloc.width / 2;
 
@@ -691,11 +691,12 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 	gint x, y, width, total;
 	gint button_width;
 	gdouble num, i;
+	guint next_possible = 0;
 
 	BraseroProjectSize *self;
 	gdouble fraction = 0.0;
 	gint text_height = 0;
-	BraseroDrive *drive;
+	BraseroDriveSize *drive;
 
 	gint bar_width, bar_height;
 	gchar *markup;
@@ -719,19 +720,18 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 					self->priv->arrow,
 					event);
 
-	if (!is_rtl)
-		button_width = self->priv->button->allocation.width;
-	else
-		button_width = 0;
+	button_width = self->priv->button->allocation.width;
 
 	drive = self->priv->current;
 	if (!drive)
 		return FALSE;
 
-	/* the number of interval needs to be reasonable, not over 8 not under 5 */
+	/* The number of interval needs to be reasonable, not over 8 not under 5
+	 * They should also depend on the available space for the bar. */
+	
 	if (self->priv->is_audio_context)
 		interval_size = AUDIO_INTERVAL_CD;
-	else if (self->priv->current->media & BRASERO_MEDIUM_DVD)
+	else if (self->priv->current->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
 		interval_size = DATA_INTERVAL_DVD;
 	else
 		interval_size = DATA_INTERVAL_CD;
@@ -746,13 +746,20 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 	} while (num > MAX_INTERVAL || num < MIN_INTERVAL);
 
 	/* calculate the size of the interval in pixels */
-	bar_width = widget->allocation.width - self->priv->button->allocation.width - self->priv->frame->style->xthickness * 2;
+	bar_width = widget->allocation.width - button_width - self->priv->frame->style->xthickness * 2;
 	interval_width = bar_width / num;
 
 	/* draw the ruler */
 	layout = gtk_widget_create_pango_layout (widget, NULL);
-	for (i = 1.0; i < num; i ++) {
+
+	if (is_rtl)
+		next_possible = widget->allocation.width - button_width - self->priv->frame->style->xthickness;
+	else
+		next_possible = button_width + self->priv->frame->style->xthickness;
+
+	for (i = 1.; i < num; i ++) {
 		gchar *string;
+		guint text_x;
 
 		string = brasero_utils_get_sectors_string (i * interval_size,
 							   self->priv->is_audio_context,
@@ -770,12 +777,28 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 		y = widget->allocation.height - extents.height;
 
 		if (is_rtl)
-			x = widget->allocation.width - self->priv->frame->style->xthickness - i * interval_width + widget->style->xthickness + ARROW_WIDTH / 2;
+			x = widget->allocation.width - button_width - self->priv->frame->style->xthickness -
+			    i * interval_width;
 		else
-			x = i * interval_width + self->priv->frame->style->xthickness - widget->style->xthickness - extents.width - ARROW_WIDTH / 2;
+			x = button_width + self->priv->frame->style->xthickness + i * interval_width;
+
+		if (!is_rtl && x <= next_possible)
+			continue;
+
+		if (is_rtl && x >= next_possible)
+			continue;
+
+		if (is_rtl)
+			text_x = x + widget->style->xthickness + ARROW_WIDTH / 2 + 2;
+		else
+			text_x = x - widget->style->xthickness - ARROW_WIDTH / 2 - extents.width - 2;
+
+		if (!is_rtl && text_x <= next_possible)
+			continue;
+
+		if (is_rtl && text_x + extents.width >= next_possible)
+			continue;
 
-		/* add the button width */
-		x += button_width;
 		gtk_paint_layout (widget->style,
 				  widget->window,
 				  GTK_STATE_NORMAL,
@@ -783,20 +806,11 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 				  &event->area,
 				  widget,
 				  NULL,
-				  x,
+				  text_x,
 				  y,
 				  layout);
-	}
-	g_object_unref (layout);
 
-	for (i = 1.0; i < num; i ++) {
-		if (is_rtl)
-			x = widget->allocation.width - self->priv->frame->style->xthickness - i * interval_width - ARROW_WIDTH / 2;
-		else
-			x = i * interval_width + self->priv->frame->style->xthickness - ARROW_WIDTH / 2;
-
-		x += button_width;
-  		gtk_paint_arrow (widget->style,
+		gtk_paint_arrow (widget->style,
 				 widget->window,
 				 GTK_STATE_NORMAL,
 				 GTK_SHADOW_ETCHED_IN,
@@ -805,21 +819,29 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 				 NULL,
 				 GTK_ARROW_UP,
 				 FALSE,
-				 x,
+				 x - widget->style->xthickness - ARROW_WIDTH/ 2,
 				 widget->allocation.height - text_height,
 				 ARROW_WIDTH,
 				 ARROW_HEIGHT);
+
+		/* calculate the next possible location (2 pixels spacing) */
+		if (is_rtl)
+			next_possible = x - ARROW_WIDTH / 2;
+		else
+			next_possible = x + 2 + ARROW_WIDTH / 2;
 	}
+	g_object_unref (layout);
 
-	bar_height = widget->allocation.height - text_height;
+	bar_height = widget->allocation.height - text_height - 2;
 
-	if (drive->drive
-	&& (NCB_MEDIA_GET_STATUS (drive->drive) & BRASERO_MEDIUM_REWRITABLE)
+	if (drive->medium
+	&& (brasero_medium_get_status (drive->medium) & BRASERO_MEDIUM_REWRITABLE)
 	&& !self->priv->multi)	
 		disc_size = drive->sectors;
 	else
 		disc_size = drive->free_space;
 
+	/* green part */
 	fraction = ((gdouble) self->priv->sectors / (gdouble) disc_size);
 	if (fraction > 1.0)
 		width = bar_width / fraction * 1.0;
@@ -827,11 +849,10 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 		width = fraction * bar_width;
 
 	if (is_rtl)
-		x = widget->allocation.width - width - self->priv->frame->style->xthickness;
+		x = self->priv->frame->style->xthickness + bar_width - width;
 	else
-		x = self->priv->frame->style->xthickness;
+		x = self->priv->frame->style->xthickness + button_width;
 
-	x += button_width;
 	gtk_paint_flat_box (widget->style,
 			    widget->window,
 			    GTK_STATE_INSENSITIVE,
@@ -853,11 +874,10 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 			width2 = bar_width / fraction * (fraction - 1.0);
 
 		if (is_rtl)
-			x = widget->allocation.width - width - width2 - self->priv->frame->style->xthickness;
+			x = widget->allocation.width - width - width2 - self->priv->frame->style->xthickness - button_width;
 		else
-			x = width + self->priv->frame->style->xthickness;
+			x = width + self->priv->frame->style->xthickness + button_width;
 
-		x += button_width;
 		gtk_paint_flat_box (widget->style,
 				    widget->window,
 				    GTK_STATE_ACTIVE,
@@ -868,16 +888,15 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 				    x,
 				    self->priv->frame->style->ythickness,
 				    width2,
-				    bar_height - self->priv->frame->style->ythickness);
+				    bar_height - self->priv->frame->style->ythickness * 2);
 
 		if (fraction > 1.03) {
 			if (is_rtl)
-				x = widget->allocation.width - bar_width - self->priv->frame->style->xthickness;
+				x = widget->allocation.width - bar_width - self->priv->frame->style->xthickness - button_width;
 			else
-				x = width + width2 + self->priv->frame->style->xthickness;
+				x = width + width2 + self->priv->frame->style->xthickness + button_width;
 
-		x += button_width;
-		gtk_paint_flat_box (widget->style,
+			gtk_paint_flat_box (widget->style,
 					    widget->window,
 					    GTK_STATE_PRELIGHT,
 					    GTK_SHADOW_NONE,
@@ -891,12 +910,12 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 		}
 	}
 	else {
+		/* This is the white part */
 		if (is_rtl)
-			x = width ? widget->allocation.width - bar_width + width - self->priv->frame->style->xthickness:widget->allocation.width - bar_width - self->priv->frame->style->xthickness;
+			x = self->priv->frame->style->xthickness;
 		else
-			x = width ? width:self->priv->frame->style->xthickness;
+			x = width ? width + button_width:self->priv->frame->style->xthickness + button_width;
 
-		x += button_width;
 		gtk_paint_flat_box (widget->style,
 				    widget->window,
 				    GTK_STATE_SELECTED,
@@ -910,7 +929,12 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 				    bar_height - self->priv->frame->style->ythickness);
 	}
 
-	alloc.x = button_width;
+	/* Frame around bar */
+	if (is_rtl)
+		alloc.x = 0;
+	else
+		alloc.x = button_width;
+
 	alloc.y = 0;
 	alloc.width = bar_width + self->priv->frame->style->xthickness * 2;
 	alloc.height = bar_height;
@@ -920,8 +944,17 @@ brasero_project_size_expose (GtkWidget *widget, GdkEventExpose *event)
 					event);
 
 	/* set the text */
+	pango_layout_set_width (self->priv->text_layout, -1);
 	pango_layout_get_pixel_extents (self->priv->text_layout, NULL, &extents);
-	x = (widget->allocation.width - button_width - extents.width) / 2;
+	if (extents.width > bar_width) {
+		pango_layout_set_ellipsize (self->priv->text_layout, PANGO_ELLIPSIZE_END);
+		pango_layout_set_width (self->priv->text_layout, (bar_width - BRASERO_PROJECT_SIZE_SPACE * 2) * PANGO_SCALE);
+	}
+	else
+		pango_layout_set_ellipsize (self->priv->text_layout, PANGO_ELLIPSIZE_NONE);
+
+	pango_layout_get_pixel_extents (self->priv->text_layout, NULL, &extents);
+	x = (bar_width - extents.width) / 2;
 	y = (widget->allocation.height - extents.height - text_height) / 2;
 
 	if (!is_rtl)
@@ -954,7 +987,7 @@ static void
 brasero_project_size_disc_changed_cb (GtkMenuItem *item,
 				      BraseroProjectSize *self)
 {
-	BraseroDrive *drive;
+	BraseroDriveSize *drive;
 
 	drive = g_object_get_data (G_OBJECT (item), DRIVE_STRUCT);
 	self->priv->current = drive;
@@ -983,10 +1016,7 @@ brasero_project_size_menu_position_cb (GtkMenu *menu,
 	gdk_drawable_get_size (GDK_DRAWABLE (self->window), &width, &height);
 	gtk_widget_set_size_request (GTK_WIDGET (menu), width, -1);
 
-	if (gtk_widget_get_direction (self) == GTK_TEXT_DIR_LTR)
-		*x = sx;
-	else
-		*x = sx - req.width;
+	*x = sx;
 	*y = sy;
 
 	screen = gtk_widget_get_screen (self);
@@ -1015,7 +1045,7 @@ brasero_project_size_build_menu (BraseroProjectSize *self)
 
 	separator = TRUE;
 	for (iter = self->priv->drives; iter; iter = iter->next) {
-		BraseroDrive *drive;
+		BraseroDriveSize *drive;
 		GtkWidget *image;
 		gchar *size_str;
 		gchar *label;
@@ -1031,19 +1061,19 @@ brasero_project_size_build_menu (BraseroProjectSize *self)
 			continue;
 
 		if (self->priv->is_audio_context
-		&& (drive->media & BRASERO_MEDIUM_DVD))
+		&& (drive->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)))
 			continue;
 
-		if (!drive->drive && !separator) {
+		if (!drive->medium && !separator) {
 			item = gtk_separator_menu_item_new ();
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			separator = TRUE;
 		}
-		else if (drive->drive)
+		else if (drive->medium)
 			separator = FALSE;
 
-		if (drive->drive
-		&& (NCB_MEDIA_GET_STATUS (drive->drive) & BRASERO_MEDIUM_REWRITABLE)
+		if (drive->medium
+		&& (brasero_medium_get_status (drive->medium) & BRASERO_MEDIUM_REWRITABLE)
 		&& !self->priv->multi)
 			disc_size = drive->sectors;
 		else
@@ -1054,12 +1084,14 @@ brasero_project_size_build_menu (BraseroProjectSize *self)
 		else
 			size_str = brasero_utils_get_size_string (disc_size * DATA_SECTOR_SIZE, TRUE, TRUE); 
 
-		if (drive->drive)
-			label = g_strdup_printf (_("%s (%s) inserted in %s"),
-						 size_str,
-						 NCB_MEDIA_GET_TYPE_STRING (drive->drive),
-						 nautilus_burn_drive_get_name_for_display (drive->drive));
-		else if (drive->media & BRASERO_MEDIUM_DL)
+		if (drive->medium) {
+			gchar *name;
+
+			name = brasero_volume_get_display_label (BRASERO_VOLUME (drive->medium), FALSE);
+			label = g_strdup_printf ("%s %s", size_str, name);
+			g_free (name);
+		}
+		else if (drive->media & BRASERO_MEDIUM_DVD_DL)
 			label = g_strdup_printf (_("%s (DVD-R Dual Layer)"),
 						 size_str);
 		else if (drive->media & BRASERO_MEDIUM_DVD)
@@ -1071,12 +1103,33 @@ brasero_project_size_build_menu (BraseroProjectSize *self)
 
 		g_free (size_str);
 
-		item = gtk_image_menu_item_new_with_label (label);
-		g_free (label);
+		if (self->priv->current == drive) {
+			gchar *tmp;
+			GtkWidget *widget;
 
-		if (!drive->drive)
+			/* This is the selected drive mark it as such */
+			tmp = g_strdup_printf ("<b><i>%s</i></b>", label);
+			g_free (label);
+			label = tmp;
+
+			widget = gtk_label_new (label);
+			gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
+			gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
+			gtk_label_set_ellipsize (GTK_LABEL (widget), PANGO_ELLIPSIZE_END);
+			g_free (label);
+
+			item = gtk_image_menu_item_new ();
+			gtk_item_select (GTK_ITEM (item));
+			gtk_container_add (GTK_CONTAINER (item), widget);
+		}
+		else {
+			item = gtk_image_menu_item_new_with_label (label);
+			g_free (label);
+		}
+
+		if (!drive->medium)
 			image = gtk_image_new_from_icon_name ("drive-optical", GTK_ICON_SIZE_MENU);
-		else if (drive->media & BRASERO_MEDIUM_DVD)
+		else if (drive->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
 			image = gtk_image_new_from_icon_name ("gnome-dev-disc-dvdr", GTK_ICON_SIZE_MENU);
 		else
 			image = gtk_image_new_from_icon_name ("gnome-dev-disc-cdr", GTK_ICON_SIZE_MENU);
@@ -1158,41 +1211,51 @@ brasero_project_size_scroll_event (GtkWidget *widget,
 				   GdkEventScroll *event)
 {
 	BraseroProjectSize *self;
+	BraseroBurnCaps *caps;
 
 	self = BRASERO_PROJECT_SIZE (widget);
+	caps = brasero_burn_caps_get_default ();
+
 	if (event->direction == GDK_SCROLL_DOWN) {
 		GList *node, *iter;
 
 		node = g_list_find (self->priv->drives, self->priv->current);
 		iter = g_list_next (node);
-		if (!iter)
+		if (!iter) {
+			g_object_unref (caps);
 			return TRUE;
+		}
 
 		while (iter != node) {
-			BraseroDrive *drive;
+			BraseroDriveSize *drive;
+			BraseroMedia media_status;
 
 			drive = iter->data;
+			media_status = brasero_burn_caps_media_capabilities (caps, drive->media);
 
 			/* must be a valid media */
 			if (!BRASERO_MEDIUM_VALID (drive->media))
 				iter = g_list_next (iter);
 			/* in an audio context only CDs are valid */
 			else if (self->priv->is_audio_context
-			     && (drive->media & BRASERO_MEDIUM_DVD))
+			     && (drive->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)))
 				iter = g_list_next (iter);
-			/* the media must be writable or rewritable or (in a
-			 * data context) at least appendable */
-			else if (!(drive->media & (BRASERO_MEDIUM_BLANK|BRASERO_MEDIUM_REWRITABLE))
-			     && ( self->priv->is_audio_context
-			     ||  !(drive->media & BRASERO_MEDIUM_APPENDABLE)))
+			/* we want a drive supported by the library */
+			else if (!(media_status & (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_REWRITABLE)))
+				iter = g_list_next (iter);
+			/* if we are in an audio context no drive with data */
+			else if (self->priv->is_audio_context
+			     &&  drive->media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA))
 				iter = g_list_next (iter);
 			else {
 				self->priv->current = drive;
 				break;
 			}
 
-			if (!iter)
+			if (!iter) {
+				g_object_unref (caps);
 				return TRUE;
+			}
 		}
 		brasero_project_size_disc_changed (self);
 	}
@@ -1201,38 +1264,46 @@ brasero_project_size_scroll_event (GtkWidget *widget,
 
 		node = g_list_find (self->priv->drives, self->priv->current);
 		iter = g_list_previous (node);
-		if (!iter)
+		if (!iter) {
+			g_object_unref (caps);
 			return TRUE;
+		}
 
 		while (iter != node) {
-			BraseroDrive *drive;
+			BraseroDriveSize *drive;
+			BraseroMedia media_status;
 
 			drive = iter->data;
+			media_status = brasero_burn_caps_media_capabilities (caps, drive->media);
 
 			/* must be a valid media */
 			if (!BRASERO_MEDIUM_VALID (drive->media))
 				iter = g_list_previous (iter);
 			/* in an audio context only CDs are valid */
 			else if (self->priv->is_audio_context
-			     && (drive->media & BRASERO_MEDIUM_DVD))
+			     && (drive->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)))
 				iter = g_list_previous (iter);
-			/* the media must be writable or rewritable or (in a
-			 * data context) at least appendable */
-			else if (!(drive->media & (BRASERO_MEDIUM_BLANK|BRASERO_MEDIUM_REWRITABLE))
-			     && (self->priv->is_audio_context
-			     ||!(drive->media & BRASERO_MEDIUM_APPENDABLE)))
+			/* we want a drive supported by the library */
+			else if (!(media_status & (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_REWRITABLE)))
+				iter = g_list_previous (iter);
+			/* if we are in an audio context no drive with data */
+			else if (self->priv->is_audio_context
+			     &&  drive->media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA))
 				iter = g_list_previous (iter);
 			else {
 				self->priv->current = drive;
 				break;
 			}
 
-			if (!iter)
+			if (!iter) {
+				g_object_unref (caps);
 				return TRUE;
+			}
 		}
 		brasero_project_size_disc_changed (self);
 	}
 
+	g_object_unref (caps);
 	return FALSE;
 }
 
@@ -1282,28 +1353,38 @@ static void
 brasero_project_size_find_proper_drive (BraseroProjectSize *self)
 {
 	GList *iter;
-	BraseroDrive *candidate = NULL;
+	BraseroBurnCaps *caps;
+	BraseroMedia media_status;
+	BraseroDriveSize *candidate = NULL;
+
+	caps = brasero_burn_caps_get_default ();
 
 	if (self->priv->current) {
-		BraseroDrive *current;
+		BraseroDriveSize *current;
 
 		/* we check the current drive to see if it is suitable */
 		current = self->priv->current;
+		media_status = brasero_burn_caps_media_capabilities (caps, current->media);
 
 		if (self->priv->is_audio_context
-		&& (current->media & BRASERO_MEDIUM_DVD)) {
+		&& (current->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))) {
 			current = NULL;
 		}
 		else if (!BRASERO_MEDIUM_VALID (current->media)) {
 			current = NULL;
 		}
-	    	else if (!(current->media & (BRASERO_MEDIUM_BLANK|BRASERO_MEDIUM_REWRITABLE))
-		     && (!(current->media & BRASERO_MEDIUM_APPENDABLE)
-		     ||   self->priv->is_audio_context)) {
+		else if (!(media_status & (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_REWRITABLE))) {
+			/* we want a drive supported by the library */
 			current = NULL;
 		}
-		else if (current->sectors >= self->priv->sectors && current->drive) {
+	    	else if (self->priv->is_audio_context
+		     &&  current->media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) {
+			/* if we are in an audio context no drive with data */
+			current = NULL;
+		}
+		else if (current->sectors >= self->priv->sectors && current->medium) {
 			/* The current drive is still a perfect fit keep it */
+			g_object_unref (caps);
 			return;
 		}
 		else if (self->priv->multi) {
@@ -1315,6 +1396,7 @@ brasero_project_size_find_proper_drive (BraseroProjectSize *self)
 			 * - we don't change the current drive if it is real
 			 * unless another real drive comes up with a size
 			 * fitting the size of the selection. */
+			g_object_unref (caps);
 			return;
 		}
 		else /* see if there is better */
@@ -1323,21 +1405,26 @@ brasero_project_size_find_proper_drive (BraseroProjectSize *self)
 
 	/* Try to find the first best candidate */
 	for (iter = self->priv->drives; iter; iter = iter->next) {
-		BraseroDrive *drive;
+		BraseroDriveSize *drive;
 
 		drive = iter->data;
 
 		/* No DVD if context is audio */
 		if (self->priv->is_audio_context
-		&& (drive->media & BRASERO_MEDIUM_DVD))
+		&& (drive->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)))
 			continue;
 
 		if (!BRASERO_MEDIUM_VALID (drive->media))
 			continue;
 
-	    	if (!(drive->media & (BRASERO_MEDIUM_BLANK|BRASERO_MEDIUM_REWRITABLE))
-		&& (!(drive->media & BRASERO_MEDIUM_APPENDABLE)
-		||   self->priv->is_audio_context))
+		/* we want a drive supported by the library */
+		media_status = brasero_burn_caps_media_capabilities (caps, drive->media);
+		if (!(media_status & (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_REWRITABLE)))
+			continue;
+
+		/* if we are in an audio context no drive with data */
+	    	if (self->priv->is_audio_context
+		&&  drive->media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA))
 			continue;
 
 		/* we must have at least one candidate */
@@ -1349,22 +1436,24 @@ brasero_project_size_find_proper_drive (BraseroProjectSize *self)
 			continue;
 
 		if (candidate->sectors < self->priv->sectors) {
-			if (!candidate->drive) {
+			if (!candidate->medium) {
 				candidate = drive;
 
-				if (drive->drive)
+				if (drive->medium)
 					break;
 			}
-			else if (drive->drive) {
+			else if (drive->medium) {
 				candidate = drive;
 				break;
 			}
 		}
-		else if (drive->drive) {
+		else if (drive->medium) {
 			candidate = drive;
 			break;
 		}
 	}
+
+	g_object_unref (caps);
 	self->priv->current = candidate;
 }
 
@@ -1372,7 +1461,7 @@ void
 brasero_project_size_set_context (BraseroProjectSize *self,
 				  gboolean is_audio)
 {
-	BraseroDrive *current;
+	BraseroDriveSize *current;
 
 	self->priv->sectors = 0;
 	self->priv->is_audio_context = is_audio;
@@ -1390,10 +1479,10 @@ brasero_project_size_set_context (BraseroProjectSize *self,
 	current = self->priv->current;
 	if (!current)
 		brasero_project_size_find_proper_drive (self);
-	else if (!current->drive)
+	else if (!current->medium)
 		brasero_project_size_find_proper_drive (self);
 	else if (is_audio
-	     && (current->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_APPENDABLE)))
+	     && (current->media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL|BRASERO_MEDIUM_APPENDABLE)))
 		brasero_project_size_find_proper_drive (self);
 
 	brasero_project_size_disc_changed (self);
@@ -1417,8 +1506,8 @@ brasero_project_size_check_status (BraseroProjectSize *self,
 	if (!self->priv->current)
 		return FALSE;
 
-	if (self->priv->current->drive
-	&& (NCB_MEDIA_GET_STATUS (self->priv->current->drive) & BRASERO_MEDIUM_REWRITABLE)
+	if (self->priv->current->medium
+	&& (brasero_medium_get_status (self->priv->current->medium) & BRASERO_MEDIUM_REWRITABLE)
 	&& !self->priv->multi)
 		disc_size = self->priv->current->sectors;
 	else
@@ -1464,32 +1553,29 @@ brasero_project_size_check_status (BraseroProjectSize *self,
 
 /********************************* real drives *********************************/
 static void
-brasero_project_size_disc_added_cb (NautilusBurnDriveMonitor *monitor,
-				    NautilusBurnDrive *ndrive,
+brasero_project_size_disc_added_cb (BraseroMediumMonitor *monitor,
+				    BraseroMedium *medium,
 				    BraseroProjectSize *self)
 {
-	GList *iter;
+	BraseroDriveSize *drive;
 
-	for (iter = self->priv->drives; iter; iter = iter->next) {
-		BraseroDrive *bdrive;
+	/* first add it to the list */
+	drive = g_new0 (BraseroDriveSize, 1);
+	drive->medium = medium;
+	g_object_ref (medium);
 
-		bdrive = iter->data;
-		if (bdrive->drive
-		&&  nautilus_burn_drive_equal (ndrive, bdrive->drive)) {
+	self->priv->drives = g_list_prepend (self->priv->drives, drive);
 
-			bdrive->media = NCB_MEDIA_GET_STATUS (bdrive->drive);
+	drive->media = brasero_medium_get_status (medium);
 
-			/* If there is an appendable session we just ignore it,
-			 * the size of this session will simply be added to the
-			 * size of the project if the user decides to merge them
-			 */
-			NCB_MEDIA_GET_CAPACITY (ndrive, NULL, &bdrive->sectors);
-			NCB_MEDIA_GET_FREE_SPACE (ndrive, NULL, &bdrive->free_space);
+	/* If there is an appendable session we just ignore it, the size of this
+	 * session will simply be added to the size of the project if the user
+	 * decides to merge them */
+	brasero_medium_get_capacity (medium, NULL, &drive->sectors);
+	brasero_medium_get_free_space (medium, NULL, &drive->free_space);
 
-			brasero_project_size_find_proper_drive (self);
-			brasero_project_size_disc_changed (self);
-		}
-	}
+	brasero_project_size_find_proper_drive (self);
+	brasero_project_size_disc_changed (self);
 
 	/* we need to rebuild the menu is any */
 	if (self->priv->menu)
@@ -1497,27 +1583,31 @@ brasero_project_size_disc_added_cb (NautilusBurnDriveMonitor *monitor,
 }
 
 static void
-brasero_project_size_disc_removed_cb (NautilusBurnDriveMonitor *monitor,
-				      NautilusBurnDrive *ndrive,
+brasero_project_size_disc_removed_cb (BraseroMediumMonitor *monitor,
+				      BraseroMedium *medium,
 				      BraseroProjectSize *self)
 {
 	GList *iter;
+	GList *next;
 
-	for (iter = self->priv->drives; iter; iter = iter->next) {
-		BraseroDrive *bdrive;
+	for (iter = self->priv->drives; iter; iter = next) {
+		BraseroDriveSize *drive;
 
-		bdrive = iter->data;
-		if (bdrive->drive
-		&&  nautilus_burn_drive_equal (ndrive, bdrive->drive)) {
-			bdrive->media = BRASERO_MEDIUM_NONE;
-			bdrive->sectors = 0;
-			bdrive->free_space = 0;
+		drive = iter->data;
+		next = iter->next;
+		if (medium == drive->medium) {
+			if (self->priv->current == drive)
+				self->priv->current = NULL;
 
-			brasero_project_size_find_proper_drive (self);
-			brasero_project_size_disc_changed (self);
+			self->priv->drives = g_list_remove (self->priv->drives, drive);
+			g_object_unref (drive->medium);
+			g_free (drive);
 		}
 	}
 
+	brasero_project_size_find_proper_drive (self);
+	brasero_project_size_disc_changed (self);
+
 	/* we need to rebuild the menu is any */
 	if (self->priv->menu)
 		brasero_project_size_show_menu_real (self, NULL);
@@ -1526,37 +1616,39 @@ brasero_project_size_disc_removed_cb (NautilusBurnDriveMonitor *monitor,
 static void
 brasero_project_size_add_real_medias (BraseroProjectSize *self)
 {
-	GList *iter, *list;
+	GSList *iter, *list;
+	BraseroMediumMonitor *monitor;
+
+	monitor = brasero_medium_monitor_get_default ();
+	g_signal_connect (monitor,
+			  "medium-added",
+			  G_CALLBACK (brasero_project_size_disc_added_cb),
+			  self);
+	g_signal_connect (monitor,
+			  "medium-removed",
+			  G_CALLBACK (brasero_project_size_disc_removed_cb),
+			  self);
+	list = brasero_medium_monitor_get_media (monitor,
+						 BRASERO_MEDIA_TYPE_WRITABLE|
+						 BRASERO_MEDIA_TYPE_REWRITABLE);
+	g_object_unref (monitor);
 
-	NCB_DRIVE_GET_LIST (list, TRUE, FALSE);
 	for (iter = list; iter; iter = iter->next) {
-		NautilusBurnDriveMonitor *monitor;
-		BraseroDrive *drive;
+		BraseroDriveSize *drive;
 
-		drive = g_new0 (BraseroDrive, 1);
-		drive->drive = iter->data;
+		drive = g_new0 (BraseroDriveSize, 1);
+		drive->medium = iter->data;
 		self->priv->drives = g_list_prepend (self->priv->drives, drive);
 
-		/* add a callback if media changes */
-		monitor = nautilus_burn_get_drive_monitor ();
-		g_signal_connect (monitor,
-				  "media-added",
-				  G_CALLBACK (brasero_project_size_disc_added_cb),
-				  self);
-		g_signal_connect (monitor,
-				  "media-removed",
-				  G_CALLBACK (brasero_project_size_disc_removed_cb),
-				  self);
-
 		/* get all the information about the current media */
-		drive->media = NCB_MEDIA_GET_STATUS (drive->drive);
+		drive->media = brasero_medium_get_status (drive->medium);
 		if (!BRASERO_MEDIUM_VALID (drive->media))
 			continue;
 
-		NCB_MEDIA_GET_CAPACITY (drive->drive, NULL, &drive->sectors);
-		NCB_MEDIA_GET_FREE_SPACE (drive->drive, NULL, &drive->free_space);
+		brasero_medium_get_capacity (drive->medium, NULL, &drive->sectors);
+		brasero_medium_get_free_space (drive->medium, NULL, &drive->free_space);
 	}
-	g_list_free (list);
+	g_slist_free (list);
 
 	brasero_project_size_find_proper_drive (self);
 	brasero_project_size_disc_changed (self);
@@ -1568,18 +1660,18 @@ brasero_project_get_ruler_height (BraseroProjectSize *self)
 	return self->priv->ruler_height;
 }
 
-NautilusBurnDrive *
-brasero_project_size_get_active_drive (BraseroProjectSize *self)
+BraseroMedium *
+brasero_project_size_get_active_medium (BraseroProjectSize *self)
 {
-	NautilusBurnDrive *drive;
+	BraseroMedium *medium;
 
 	if (!self->priv->current)
 		return NULL;
 
-	if (!self->priv->current->drive)
+	if (!self->priv->current->medium)
 		return NULL;
 
-	drive = self->priv->current->drive;
-	nautilus_burn_drive_ref (drive);
-	return drive;
+	medium = self->priv->current->medium;
+	g_object_ref (medium);
+	return medium;
 }
