@@ -36,6 +36,8 @@
 #include <glib/gutils.h>
 #include <glib-object.h>
 
+#include <gio/gio.h>
+
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkfilefilter.h>
 #include <gtk/gtkselection.h>
@@ -57,10 +59,6 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkmessagedialog.h>
 
-#include <libgnomeui/libgnomeui.h>
-
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-
 #include <beagle/beagle.h>
 
 #include "brasero-utils.h"
@@ -697,8 +695,10 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 	GSList *iter;
 	GSList *next;
 
-	gchar *name, *mime, *uri, *icon_string;
+	gchar *name, *mime; 
+	const gchar *icon_string = BRASERO_DEFAULT_ICON;
 	const gchar *description;
+	GIcon *icon;
 	gint score;
 	gint num;
 
@@ -707,7 +707,8 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 
 	num = 0;
 	for (iter = list; iter && num < max; iter = next, num ++) {
-		gchar *unescaped_uri;
+		gchar *unescaped_uri, *uri;
+		GFile *file;
 
 		hit = iter->data;
 		next = iter->next;
@@ -716,13 +717,15 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 
 		/* beagle return badly formed uri not encoded in UTF-8
 		 * locale charset so we check them just in case */
-		unescaped_uri = gnome_vfs_unescape_string_for_display (uri);
+		unescaped_uri = g_uri_unescape_string (uri, NULL);
 		if (!g_utf8_validate (unescaped_uri, -1, NULL)) {
 			g_free (unescaped_uri);
 			g_free (uri);
 			continue;
 		}
 
+		file = g_file_new_for_uri (uri);
+
 		name = g_path_get_basename (unescaped_uri);
 		g_free (unescaped_uri);
 
@@ -739,10 +742,28 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 			mime = g_strdup ("x-directory/normal");
 		}
 
-		description = gnome_vfs_mime_get_description (mime);
-		icon_string = gnome_icon_lookup (gtk_icon_theme_get_default (), NULL,
-						 NULL, NULL, NULL, mime,
-						 GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+		description = g_content_type_get_description (mime);
+
+		icon = g_content_type_get_icon (mime);
+		icon_string = NULL;
+		if (G_IS_THEMED_ICON (icon)) {
+			const gchar * const *names = NULL;
+
+			names = g_themed_icon_get_names (G_THEMED_ICON (icon));
+			if (names) {
+				gint i;
+				GtkIconTheme *theme;
+
+				theme = gtk_icon_theme_get_default ();
+				for (i = 0; names [i]; i++) {
+					if (gtk_icon_theme_has_icon (theme, names [i])) {
+						icon_string = names [i];
+						break;
+					}
+				}
+			}
+		}
+
 		score = (int) (beagle_hit_get_score (hit) * 100);
 
 		gtk_list_store_append (GTK_LIST_STORE (model), &row);
@@ -756,10 +777,9 @@ brasero_search_add_hit_to_tree (BraseroSearch *search,
 				    -1);
 
 		/* add the mime type to the filter combo */
-		brasero_mime_filter_add_mime (BRASERO_MIME_FILTER (search->priv->filter),
-					      mime);
+		brasero_mime_filter_add_mime (BRASERO_MIME_FILTER (search->priv->filter), mime);
 
-		g_free (icon_string);
+		g_object_unref (icon);
 		g_free (name);
 		g_free (mime);
 		g_free (uri);
@@ -1031,7 +1051,7 @@ brasero_search_beagle_error_dialog (BraseroSearch *search, GError *error)
 					 GTK_BUTTONS_CLOSE,
 					 _("Error querying beagle:"));
 
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Search error"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Search Error"));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  error->message);
