@@ -49,12 +49,11 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkmessagedialog.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-#include <nautilus-burn-drive.h>
-
 #include "brasero-sum-dialog.h"
 #include "brasero-tool-dialog.h"
-#include "brasero-ncb.h"
+#include "burn-drive.h"
+#include "burn-medium.h"
+#include "burn-volume-obj.h"
 #include "brasero-xfer.h"
 #include "burn-basics.h"
 #include "burn-debug.h"
@@ -72,7 +71,7 @@ struct _BraseroSumDialogPrivate {
 	BraseroXferCtx *xfer_ctx;
 };
 
-static GtkDialogClass *parent_class = NULL;
+static BraseroToolDialogClass *parent_class = NULL;
 
 static void
 brasero_sum_dialog_md5_toggled (GtkToggleButton *button,
@@ -150,7 +149,7 @@ brasero_sum_dialog_message_error (BraseroSumDialog *self,
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File integrity check error"),
+					   _("File Integrity Check Error"),
 					   _("The file integrity check cannot be performed:"),
 					   error ? error->message:_("unknown error"),
 					   GTK_MESSAGE_ERROR);
@@ -164,7 +163,7 @@ brasero_sum_dialog_success (BraseroSumDialog *self)
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File integrity check success"),
+					   _("File Integrity Check Success"),
 					   _("The file integrity was performed successfully:"),
 					   _("there seems to be no corrupted file on the disc."),
 					   GTK_MESSAGE_INFO);
@@ -196,7 +195,7 @@ brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
 						      GTK_BUTTONS_NONE,
 						      _("<b><big>The following files appear to be corrupted:</big></b>"));
 
-	gtk_window_set_title (GTK_WINDOW (message),  _("File integrity check error"));
+	gtk_window_set_title (GTK_WINDOW (message),  _("File Integrity Check Error"));
 	gtk_window_set_resizable (GTK_WINDOW (message), TRUE);
 	gtk_widget_set_size_request (GTK_WIDGET (message), 440, 300);
 
@@ -291,14 +290,12 @@ brasero_sum_dialog_progress_poll (gpointer user_data)
 
 static BraseroBurnResult
 brasero_sum_dialog_download (BraseroSumDialog *self,
-			     GnomeVFSURI *vfsuri,
+			     const gchar *src,
 			     gchar **retval,
 			     GError **error)
 {
 	BraseroBurnResult result;
-	GnomeVFSURI *tmpuri;
 	gchar *tmppath;
-	gchar *uri;
 	gint id;
 	int fd;
 
@@ -316,21 +313,6 @@ brasero_sum_dialog_download (BraseroSumDialog *self,
 	}
 	close (fd);
 
-	uri = gnome_vfs_get_uri_from_local_path (tmppath);
-	if (!uri) {
-		g_remove (tmppath);
-		g_free (tmppath);
-
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("URI is not valid"));
-		return BRASERO_BURN_ERR;
-	}
-
-	tmpuri = gnome_vfs_uri_new (uri);
-	g_free (uri);
-
 	brasero_tool_dialog_set_action (BRASERO_TOOL_DIALOG (self),
 					BRASERO_BURN_ACTION_FILE_COPY,
 					_("Downloading md5 file"));
@@ -341,12 +323,10 @@ brasero_sum_dialog_download (BraseroSumDialog *self,
 
 	self->priv->xfer_ctx = brasero_xfer_new ();
 	result = brasero_xfer (self->priv->xfer_ctx,
-			       vfsuri,
-			       tmpuri,
+			       src,
+			       tmppath,
 			       error);
 
-	gnome_vfs_uri_unref (tmpuri);
-
 	g_source_remove (id);
 	brasero_xfer_free (self->priv->xfer_ctx);
 	self->priv->xfer_ctx = NULL;
@@ -368,17 +348,18 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 				      GError **error)
 {
 	BraseroBurnResult result;
-	GnomeVFSURI *vfsuri;
 	gchar buffer [33];
+	GFile *file_src;
 	gchar *tmppath;
+	gchar *scheme;
 	gchar *uri;
 	gchar *src;
 	FILE *file;
 	int read;
 
 	/* see if this file needs downloading */
-	uri = gnome_vfs_make_uri_from_input (file_path);
-	if (!uri) {
+	file_src = g_file_new_for_commandline_arg (file_path);
+	if (!file_src) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -387,27 +368,28 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 	}
 
 	tmppath = NULL;
-	vfsuri = gnome_vfs_uri_new (uri);
-	if (!gnome_vfs_uri_is_local (vfsuri)) {
-		g_free (uri);
+	scheme = g_file_get_uri_scheme (file_src);
+	if (strcmp (scheme, "file")) {
+		uri = g_file_get_uri (file_src);
+		g_object_unref (file_src);
 
 		result = brasero_sum_dialog_download (self,
-						      vfsuri,
+						      uri,
 						      &tmppath,
 						      error);
-		gnome_vfs_uri_unref (vfsuri);
-
-		if (result != BRASERO_BURN_CANCEL)
+		if (result != BRASERO_BURN_CANCEL) {
+			g_object_unref (file_src);
+			g_free (scheme);
 			return result;
+		}
 
 		src = tmppath;
 	}
 	else {
-		src = gnome_vfs_get_local_path_from_uri (uri);
-		g_free (uri);
-
-		gnome_vfs_uri_unref (vfsuri);
+		src = g_file_get_path (file_src);
+		g_object_unref (file_src);
 	}
+	g_free (scheme);
 
 	/* now get the md5 sum from the file */
 	file = fopen (src, "r");
@@ -451,7 +433,7 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 
 static BraseroBurnResult
 brasero_sum_dialog_get_disc_checksum (BraseroSumDialog *self,
-				      NautilusBurnDrive *drive,
+				      BraseroDrive *drive,
 				      gchar *checksum,
 				      GError **error)
 {
@@ -476,7 +458,7 @@ brasero_sum_dialog_get_disc_checksum (BraseroSumDialog *self,
 
 static gboolean
 brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
-				   NautilusBurnDrive *drive)
+				   BraseroMedium *medium)
 {
 	BraseroBurnResult result;
 	gchar *file_sum = NULL;
@@ -488,7 +470,7 @@ brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
     	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (self->priv->md5_chooser));
 	if (!uri) {
 		retval = brasero_sum_dialog_message (self,
-						     _("File integrity check error"),
+						     _("File Integrity Check Error"),
 						     _("The file integrity check cannot be performed:"),
 						     error ? error->message:_("no md5 file was given."),
 						     GTK_MESSAGE_ERROR);
@@ -510,7 +492,10 @@ brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
 		return retval;
 	}
 
-	result = brasero_sum_dialog_get_disc_checksum (self, drive, file_sum, &error);
+	result = brasero_sum_dialog_get_disc_checksum (self,
+						       brasero_medium_get_drive (medium),
+						       file_sum,
+						       &error);
 	if (result == BRASERO_BURN_CANCEL) {
 		g_free (file_sum);
 		return FALSE;
@@ -530,10 +515,76 @@ brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
 	return brasero_sum_dialog_success (self);
 }
 
+static BraseroChecksumType
+brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
+					    BraseroDrive *drive,
+					    BraseroTrack *track,
+					    GError **error)
+{
+	BraseroMedium *medium;
+	gboolean retval;
+	gchar *filename;
+	gchar *root;
+
+	medium = brasero_drive_get_medium (drive);
+
+	retval = brasero_volume_mount (BRASERO_VOLUME (medium), TRUE, error);
+	if (!retval)
+		return BRASERO_CHECKSUM_NONE;
+
+	root = brasero_volume_get_mount_point (BRASERO_VOLUME (medium), error);
+	if (!root)
+		return BRASERO_CHECKSUM_NONE;
+
+	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_MD5_FILE, NULL);
+	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		g_free (filename);
+
+		brasero_track_set_checksum (track,
+					    BRASERO_CHECKSUM_MD5_FILE,
+					    BRASERO_MD5_FILE);
+
+		return BRASERO_CHECKSUM_MD5_FILE;
+	}
+	g_free (filename);
+
+	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_SHA1_FILE, NULL);
+	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		g_free (filename);
+
+		brasero_track_set_checksum (track,
+					    BRASERO_CHECKSUM_SHA1_FILE,
+					    BRASERO_SHA1_FILE);
+
+		return BRASERO_CHECKSUM_SHA1_FILE;
+	}
+	g_free (filename);
+
+	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_SHA256_FILE, NULL);
+	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		g_free (filename);
+
+		brasero_track_set_checksum (track,
+					    BRASERO_CHECKSUM_SHA256_FILE,
+					    BRASERO_SHA256_FILE);
+
+		return BRASERO_CHECKSUM_SHA256_FILE;
+	}
+	g_free (filename);
+
+	g_set_error (error,
+		     BRASERO_ERROR,
+		     BRASERO_ERROR_GENERAL,
+		     _("no checksum file could be found on the disc."));
+
+	return BRASERO_CHECKSUM_NONE;
+}
+
 static gboolean
 brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
-				   NautilusBurnDrive *drive)
+				   BraseroDrive *drive)
 {
+	BraseroChecksumType checksum_type;
 	GSList *wrong_sums = NULL;
 	BraseroBurnResult result;
 	GError *error = NULL;
@@ -544,9 +595,19 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 	/* get the checksum */
 	track = brasero_track_new (BRASERO_TRACK_TYPE_DISC);
 	brasero_track_set_drive_source (track, drive);
-	brasero_track_set_checksum (track,
-				    BRASERO_CHECKSUM_MD5_FILE,
-				    BRASERO_MD5_FILE);
+
+	checksum_type = brasero_sum_dialog_set_track_checksum_type (self,
+								    drive,
+								    track,
+								    &error);
+
+	if (checksum_type == BRASERO_CHECKSUM_NONE) {
+		brasero_sum_dialog_message_error (self, error);
+		brasero_track_unref (track);
+		g_error_free (error);
+		return FALSE;
+	}
+
 	brasero_burn_session_add_track (self->priv->session, track);
 
 	/* It's good practice to unref the track afterwards as we don't need it
@@ -587,7 +648,7 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 
 static gboolean
 brasero_sum_dialog_activate (BraseroToolDialog *dialog,
-			     NautilusBurnDrive *drive)
+			     BraseroMedium *medium)
 {
 	BraseroSumDialog *self;
 	gboolean result;
@@ -595,14 +656,24 @@ brasero_sum_dialog_activate (BraseroToolDialog *dialog,
 	self = BRASERO_SUM_DIALOG (dialog);
 
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->priv->md5_check)))
-		result = brasero_sum_dialog_check_disc_sum (self, drive);
+		result = brasero_sum_dialog_check_disc_sum (self, brasero_medium_get_drive (medium));
 	else
-		result = brasero_sum_dialog_check_md5_file (self, drive);
+		result = brasero_sum_dialog_check_md5_file (self, medium);
 
 	brasero_tool_dialog_set_valid (dialog, TRUE);
 	return result;
 }
 
+static void
+brasero_sum_dialog_drive_changed (BraseroToolDialog *dialog,
+				  BraseroMedium *medium)
+{
+	if (medium)
+		brasero_tool_dialog_set_valid (dialog, BRASERO_MEDIUM_VALID (brasero_medium_get_status (medium)));
+	else
+		brasero_tool_dialog_set_valid (dialog, FALSE);
+}
+
 static void
 brasero_sum_dialog_finalize (GObject *object)
 {
@@ -631,12 +702,14 @@ brasero_sum_dialog_class_init (BraseroSumDialogClass *klass)
 	object_class->finalize = brasero_sum_dialog_finalize;
 
 	tool_dialog_class->activate = brasero_sum_dialog_activate;
+	tool_dialog_class->drive_changed = brasero_sum_dialog_drive_changed;
 }
 
 static void
 brasero_sum_dialog_init (BraseroSumDialog *obj)
 {
 	GtkWidget *box;
+	BraseroMedium *medium;
 
 	obj->priv = g_new0 (BraseroSumDialogPrivate, 1);
 
@@ -676,6 +749,14 @@ brasero_sum_dialog_init (BraseroSumDialog *obj)
 					_("_Check"),
 					GTK_STOCK_FIND,
 					NULL);
+
+	medium = brasero_tool_dialog_get_medium (BRASERO_TOOL_DIALOG (obj));
+	if (medium) {
+		brasero_tool_dialog_set_valid (BRASERO_TOOL_DIALOG (obj), BRASERO_MEDIUM_VALID (brasero_medium_get_status (medium)));
+		g_object_unref (medium);
+	}
+	else
+		brasero_tool_dialog_set_valid (BRASERO_TOOL_DIALOG (obj), FALSE);
 }
 
 GtkWidget *
@@ -684,7 +765,7 @@ brasero_sum_dialog_new ()
 	BraseroSumDialog *obj;
 	
 	obj = BRASERO_SUM_DIALOG (g_object_new (BRASERO_TYPE_SUM_DIALOG, NULL));
-	gtk_window_set_title (GTK_WINDOW (obj), "Disc checking");
+	gtk_window_set_title (GTK_WINDOW (obj), "Disc Checking");
 	
 	return GTK_WIDGET (obj);
 }
