@@ -27,15 +27,13 @@
 
 #include <glib.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 G_BEGIN_DECLS
 
 #define BRASERO_GET_BASENAME_FOR_DISPLAY(uri, name)				\
 {										\
     	gchar *escaped_basename;						\
 	escaped_basename = g_path_get_basename (uri);				\
-    	name = gnome_vfs_unescape_string_for_display (escaped_basename);	\
+    	name = g_uri_unescape_string (escaped_basename, NULL);			\
 	g_free (escaped_basename);						\
 }
 
@@ -45,7 +43,10 @@ GQuark brasero_burn_quark (void);
 #define BRASERO_PLUGIN_DIRECTORY		BRASERO_LIBDIR "/brasero/plugins"
 
 #define BRASERO_BURN_TMP_FILE_NAME		"brasero_tmp_XXXXXX"
+
 #define BRASERO_MD5_FILE			".checksum.md5"
+#define BRASERO_SHA1_FILE			".checksum.sha1"
+#define BRASERO_SHA256_FILE			".checksum.sha256"
 
 typedef enum {
 	BRASERO_BURN_ERROR_NONE,
@@ -79,7 +80,8 @@ typedef enum {
 
 	/* these are not necessarily error */
 	BRASERO_BURN_WARNING_NEXT_COPY,
-	BRASERO_BURN_WARNING_CHECKSUM
+	BRASERO_BURN_WARNING_CHECKSUM,
+	BRASERO_BURN_WARNING_INSERT_AFTER_COPY
 } BraseroBurnError;
 
 typedef enum {
