@@ -28,8 +28,6 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <nautilus-burn-drive.h>
-
 #include "burn-basics.h"
 #include "burn-medium.h"
 #include "burn-session.h"
@@ -62,7 +60,7 @@ GType brasero_burn_caps_get_type();
 BraseroBurnCaps *brasero_burn_caps_get_default ();
 
 gint
-brasero_burn_caps_register_plugin_group (BraseroBurnCaps *self,
+brasero_burn_caps_register_plugin_group (BraseroBurnCaps *caps,
 					 const gchar *name);
 
 /**
@@ -70,9 +68,12 @@ brasero_burn_caps_register_plugin_group (BraseroBurnCaps *self,
  */
 
 BraseroMedia
-brasero_burn_caps_media_capabilities (BraseroBurnCaps *self,
+brasero_burn_caps_media_capabilities (BraseroBurnCaps *caps,
 				      BraseroMedia media);
 
+gboolean
+brasero_burn_caps_can_checksum (BraseroBurnCaps *caps);
+
 /**
  * Returns a GSList * of BraseroTask * for a given session
  */
