@@ -1,6 +1,6 @@
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
  * brasero is free software.
  * 
@@ -67,6 +67,8 @@ struct _BraseroPluginConfOption {
 		} range;
 
 		GSList *suboptions;
+
+		GSList *choices;
 	} specifics;
 };
 
@@ -126,20 +128,26 @@ void
 brasero_plugin_set_active (BraseroPlugin *self, gboolean active)
 {
 	BraseroPluginPrivate *priv;
+	gboolean was_active;
+	gboolean now_active;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
 
-	if (priv->type == G_TYPE_NONE)
-		return;
-
-	if (priv->active == active)
-		return;
-
+	was_active = brasero_plugin_get_active (self);
 	priv->active = active;
+
+	now_active = brasero_plugin_get_active (self);
+	if (was_active == now_active)
+		return;
+
+	BRASERO_BURN_LOG ("Plugin %s is %s",
+			  brasero_plugin_get_name (self),
+			  now_active?"active":"inactive");
+
 	g_signal_emit (self,
 		       plugin_signals [ACTIVATED_SIGNAL],
 		       0,
-		       active);
+		       now_active);
 }
 
 gboolean
@@ -184,6 +192,12 @@ brasero_plugin_cleanup_definition (BraseroPlugin *self)
 /**
  * Plugin configure options
  */
+static void
+brasero_plugin_conf_option_choice_pair_free (BraseroPluginChoicePair *pair)
+{
+	g_free (pair->string);
+	g_free (pair);
+}
 
 static void
 brasero_plugin_conf_option_free (BraseroPluginConfOption *option)
@@ -191,6 +205,11 @@ brasero_plugin_conf_option_free (BraseroPluginConfOption *option)
 	if (option->type == BRASERO_PLUGIN_OPTION_BOOL)
 		g_slist_free (option->specifics.suboptions);
 
+	if (option->type == BRASERO_PLUGIN_OPTION_CHOICE) {
+		g_slist_foreach (option->specifics.choices, (GFunc) brasero_plugin_conf_option_choice_pair_free, NULL);
+		g_slist_free (option->specifics.choices);
+	}
+
 	g_free (option->key);
 	g_free (option->description);
 
@@ -297,6 +316,24 @@ brasero_plugin_conf_option_int_set_range (BraseroPluginConfOption *option,
 	return BRASERO_BURN_OK;
 }
 
+BraseroBurnResult
+brasero_plugin_conf_option_choice_add (BraseroPluginConfOption *option,
+				       const gchar *string,
+				       gint value)
+{
+	BraseroPluginChoicePair *pair;
+
+	if (option->type != BRASERO_PLUGIN_OPTION_CHOICE)
+		return BRASERO_BURN_ERR;
+
+	pair = g_new0 (BraseroPluginChoicePair, 1);
+	pair->value = value;
+	pair->string = g_strdup (string);
+	option->specifics.choices = g_slist_append (option->specifics.choices, pair);
+
+	return BRASERO_BURN_OK;
+}
+
 GSList *
 brasero_plugin_conf_option_bool_get_suboptions (BraseroPluginConfOption *option)
 {
@@ -321,6 +358,14 @@ brasero_plugin_conf_option_int_get_min (BraseroPluginConfOption *option)
 	return option->specifics.range.min;
 }
 
+GSList *
+brasero_plugin_conf_option_choice_get (BraseroPluginConfOption *option)
+{
+	if (option->type != BRASERO_PLUGIN_OPTION_CHOICE)
+		return NULL;
+	return option->specifics.choices;
+}
+
 /**
  * Used to set the caps of plugin
  */
@@ -477,7 +522,9 @@ brasero_plugin_set_flags (BraseroPlugin *self,
 
 static gboolean
 brasero_plugin_get_all_flags (GSList *flags_list,
+			      gboolean check_compulsory,
 			      BraseroMedia media,
+			      BraseroBurnFlag mask,
 			      BraseroBurnFlag current,
 			      BraseroBurnFlag *supported_retval,
 			      BraseroBurnFlag *compulsory_retval)
@@ -486,7 +533,7 @@ brasero_plugin_get_all_flags (GSList *flags_list,
 	BraseroPluginFlags *flags;
 	BraseroPluginFlagPair *iter;
 	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_ALL;
+	BraseroBurnFlag compulsory = (BRASERO_BURN_FLAG_ALL & mask);
 
 	flags = brasero_plugin_get_flags (flags_list, media);
 	if (!flags) {
@@ -497,16 +544,21 @@ brasero_plugin_get_all_flags (GSList *flags_list,
 		return FALSE;
 	}
 
+	/* Find all sets of flags that support the current flags */
 	found = FALSE;
-	for (iter = flags ? flags->pairs:NULL; iter; iter = iter->next) {
+	for (iter = flags->pairs; iter; iter = iter->next) {
+		BraseroBurnFlag compulsory_masked;
+
 		if ((current & iter->supported) != current)
 			continue;
 
-		if ((current & iter->compulsory) != iter->compulsory)
+		compulsory_masked = (iter->compulsory & mask);
+		if (check_compulsory
+		&& (current & compulsory_masked) != compulsory_masked)
 			continue;
 
 		supported |= iter->supported;
-		compulsory &= iter->compulsory;
+		compulsory &= compulsory_masked;
 		found = TRUE;
 	}
 
@@ -532,26 +584,17 @@ brasero_plugin_check_record_flags (BraseroPlugin *self,
 				   BraseroBurnFlag current)
 {
 	BraseroPluginPrivate *priv;
-	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
 	current &= BRASERO_PLUGIN_BURN_FLAG_MASK;
 
-	brasero_plugin_get_all_flags (priv->flags,
-				      media,
-				      current,
-				      &supported,
-				      &compulsory);
-
-	supported &= BRASERO_PLUGIN_BURN_FLAG_MASK;
-	compulsory &= BRASERO_PLUGIN_BURN_FLAG_MASK;
-
-	if ((current & supported) == current
-	&&  (current & compulsory) == compulsory)
-		return TRUE;
-
-	return FALSE;
+	return brasero_plugin_get_all_flags (priv->flags,
+					     TRUE,
+					     media,
+					     BRASERO_PLUGIN_BURN_FLAG_MASK,
+					     current,
+					     NULL,
+					     NULL);
 }
 
 gboolean
@@ -560,29 +603,23 @@ brasero_plugin_check_image_flags (BraseroPlugin *self,
 				  BraseroBurnFlag current)
 {
 	BraseroPluginPrivate *priv;
-	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	current &= (BRASERO_BURN_FLAG_APPEND|
-		    BRASERO_BURN_FLAG_MERGE);
+	current &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 
-	brasero_plugin_get_all_flags (priv->flags,
-				      media,
-				      current,
-				      &supported,
-				      &compulsory);
-
-	supported &= (BRASERO_BURN_FLAG_APPEND|
-		      BRASERO_BURN_FLAG_MERGE);
-	compulsory &= (BRASERO_BURN_FLAG_APPEND|
-		       BRASERO_BURN_FLAG_MERGE);
-
-	if ((current & supported) == current
-	&&  (current & compulsory) == compulsory)
+	/* If there is no flag that's no use checking anything. If there is no
+	 * flag we don't care about the media and therefore it's always possible
+	 * NOTE: that's no the case for other operation like burn/blank. */
+	if (current == BRASERO_BURN_FLAG_NONE)
 		return TRUE;
 
-	return FALSE;
+	return brasero_plugin_get_all_flags (priv->flags,
+					     TRUE,
+					     media,
+					     BRASERO_PLUGIN_IMAGE_FLAG_MASK,
+					     current,
+					     NULL,
+					     NULL);
 }
 
 gboolean
@@ -599,7 +636,9 @@ brasero_plugin_get_record_flags (BraseroPlugin *self,
 	current &= BRASERO_PLUGIN_BURN_FLAG_MASK;
 
 	result = brasero_plugin_get_all_flags (priv->flags,
+					       FALSE,
 					       media,
+					       BRASERO_PLUGIN_BURN_FLAG_MASK,
 					       current,
 					       supported,
 					       compulsory);
@@ -625,11 +664,12 @@ brasero_plugin_get_image_flags (BraseroPlugin *self,
 	gboolean result;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	current &= (BRASERO_BURN_FLAG_APPEND|
-		    BRASERO_BURN_FLAG_MERGE);
+	current &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 
 	result = brasero_plugin_get_all_flags (priv->flags,
+					       FALSE,
 					       media,
+					       BRASERO_PLUGIN_IMAGE_FLAG_MASK,
 					       current,
 					       supported,
 					       compulsory);
@@ -637,11 +677,9 @@ brasero_plugin_get_image_flags (BraseroPlugin *self,
 		return FALSE;
 
 	if (supported)
-		*supported &= (BRASERO_BURN_FLAG_APPEND|
-			       BRASERO_BURN_FLAG_MERGE);
+		*supported &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 	if (compulsory)
-		*compulsory &= (BRASERO_BURN_FLAG_APPEND|
-				BRASERO_BURN_FLAG_MERGE);
+		*compulsory &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 
 	return TRUE;
 }
@@ -667,29 +705,17 @@ brasero_plugin_check_blank_flags (BraseroPlugin *self,
 				  BraseroBurnFlag current)
 {
 	BraseroPluginPrivate *priv;
-	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	current &= (BRASERO_BURN_FLAG_NOGRACE|
-		    BRASERO_BURN_FLAG_FAST_BLANK);
+	current &= BRASERO_PLUGIN_BLANK_FLAG_MASK;
 
-	brasero_plugin_get_all_flags (priv->blank_flags,
-				      media,
-				      current,
-				      &supported,
-				      &compulsory);
-
-	supported &= (BRASERO_BURN_FLAG_NOGRACE|
-		      BRASERO_BURN_FLAG_FAST_BLANK);
-	compulsory &= (BRASERO_BURN_FLAG_NOGRACE|
-		       BRASERO_BURN_FLAG_FAST_BLANK);
-
-	if ((current & supported) == current
-	&&  (current & compulsory) == compulsory)
-		return TRUE;
-
-	return FALSE;
+	return brasero_plugin_get_all_flags (priv->blank_flags,
+					     TRUE,
+					     media,
+					     BRASERO_PLUGIN_BLANK_FLAG_MASK,
+					     current,
+					     NULL,
+					     NULL);
 }
 
 gboolean
@@ -703,11 +729,12 @@ brasero_plugin_get_blank_flags (BraseroPlugin *self,
 	gboolean result;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	current &= (BRASERO_BURN_FLAG_NOGRACE|
-		    BRASERO_BURN_FLAG_FAST_BLANK);
+	current &= BRASERO_PLUGIN_BLANK_FLAG_MASK;
 
 	result = brasero_plugin_get_all_flags (priv->blank_flags,
+					       FALSE,
 					       media,
+					       BRASERO_PLUGIN_BLANK_FLAG_MASK,
 					       current,
 					       supported,
 					       compulsory);
@@ -715,11 +742,9 @@ brasero_plugin_get_blank_flags (BraseroPlugin *self,
 		return FALSE;
 
 	if (supported)
-		*supported &= (BRASERO_BURN_FLAG_NOGRACE|
-			       BRASERO_BURN_FLAG_FAST_BLANK);
+		*supported &= BRASERO_PLUGIN_BLANK_FLAG_MASK;
 	if (compulsory)
-		*compulsory &= (BRASERO_BURN_FLAG_NOGRACE|
-				BRASERO_BURN_FLAG_FAST_BLANK);
+		*compulsory &= BRASERO_PLUGIN_BLANK_FLAG_MASK;
 
 	return TRUE;
 }
@@ -987,7 +1012,7 @@ brasero_plugin_init_real (BraseroPlugin *object)
 							 NULL,
 							 NULL);
 
-	/* No need to emit notify:: signal here */
+	/* No need to emit notify:: here */
 	g_free (priority_path);
 	g_object_unref (client);
 }
