@@ -28,10 +28,9 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <nautilus-burn-drive.h>
-
 #include "burn-basics.h"
 #include "burn-track.h"
+#include "burn-drive.h"
 
 G_BEGIN_DECLS
 
@@ -96,7 +95,15 @@ void
 brasero_burn_session_set_label (BraseroBurnSession *session,
 				const gchar *label);
 
+BraseroBurnResult
+brasero_burn_session_tag_lookup (BraseroBurnSession *session,
+				 const gchar *tag,
+				 GValue **value);
 
+BraseroBurnResult
+brasero_burn_session_tag_add (BraseroBurnSession *session,
+			      const gchar *tag,
+			      GValue *value);
 /**
  * 
  */
@@ -115,7 +122,7 @@ brasero_burn_session_set_num_copies (BraseroBurnSession *session,
 guint
 brasero_burn_session_get_num_copies (BraseroBurnSession *session);
 
-NautilusBurnDrive *
+BraseroDrive *
 brasero_burn_session_get_burner (BraseroBurnSession *session);
 
 
@@ -125,7 +132,7 @@ brasero_burn_session_get_burner (BraseroBurnSession *session);
 
 void
 brasero_burn_session_set_burner (BraseroBurnSession *session,
-				 NautilusBurnDrive *burner);
+				 BraseroDrive *burner);
 
 BraseroBurnResult
 brasero_burn_session_set_image_output (BraseroBurnSession *session,
@@ -225,9 +232,12 @@ brasero_burn_session_same_src_dest_drive (BraseroBurnSession *session);
 BraseroMedia
 brasero_burn_session_get_dest_media (BraseroBurnSession *session);
 
-NautilusBurnDrive *
+BraseroDrive *
 brasero_burn_session_get_src_drive (BraseroBurnSession *session);
 
+BraseroMedium *
+brasero_burn_session_get_src_medium (BraseroBurnSession *session);
+
 gboolean
 brasero_burn_session_is_dest_file (BraseroBurnSession *session);
 
