@@ -785,6 +785,7 @@ brasero_track_get_data_paths (BraseroTrack *track,
 			      const gchar *grafts_path,
 			      const gchar *excluded_path,
 			      const gchar *emptydir,
+			      const gchar *videodir,
 			      GError **error)
 {
 	BraseroBurnResult result;
@@ -797,6 +798,7 @@ brasero_track_get_data_paths (BraseroTrack *track,
 	result = brasero_mkisofs_base_write_to_files (data->grafts,
 						      data->excluded,
 						      emptydir,
+						      videodir,
 						      grafts_path,
 						      excluded_path,
 						      error);
@@ -1019,7 +1021,7 @@ brasero_track_get_image_size (BraseroTrack *track,
 
 	image = (BraseroTrackImage *) track;
 
-	/* Convienience function */
+	/* Convenience function */
 	if (track->type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN) {
 		gchar *path;
 
