@@ -25,6 +25,7 @@
 #include <glib.h>
 
 #include "burn-basics.h"
+#include "burn-drive.h"
 #include "burn-medium.h"
 #include "burn-image-format.h"
 
@@ -49,11 +50,13 @@ typedef enum {
 	BRASERO_IMAGE_FS_JOLIET			= 1 << 2,
 	BRASERO_IMAGE_FS_VIDEO			= 1 << 3,
 	BRASERO_IMAGE_ISO_FS_LEVEL_3		= 1 << 4,
+	BRASERO_IMAGE_ISO_FS_DEEP_DIRECTORY	= 1 << 5,
 	BRASERO_IMAGE_FS_ANY			= BRASERO_IMAGE_FS_ISO|
 						  BRASERO_IMAGE_FS_UDF|
 						  BRASERO_IMAGE_FS_JOLIET|
 						  BRASERO_IMAGE_ISO_FS_LEVEL_3|
-						  BRASERO_IMAGE_FS_VIDEO
+						  BRASERO_IMAGE_FS_VIDEO|
+						  BRASERO_IMAGE_ISO_FS_DEEP_DIRECTORY
 } BraseroImageFS;
 
 typedef enum {
@@ -67,7 +70,10 @@ typedef enum {
 	BRASERO_CHECKSUM_NONE			= 0,
 	BRASERO_CHECKSUM_MD5			= 1,
 	BRASERO_CHECKSUM_MD5_FILE		= 1 << 1,
-	BRASERO_CHECKSUM_ANY			= BRASERO_CHECKSUM_MD5
+	BRASERO_CHECKSUM_SHA1			= 1 << 2,
+	BRASERO_CHECKSUM_SHA1_FILE		= 1 << 3,
+	BRASERO_CHECKSUM_SHA256			= 1 << 4,
+	BRASERO_CHECKSUM_SHA256_FILE		= 1 << 5
 } BraseroChecksumType;
 
 #define	BRASERO_MIN_AUDIO_TRACK_LENGTH		((gint64) 6 * 1000000000)
@@ -83,9 +89,6 @@ typedef enum {
 struct _BraseroGraftPt {
 	gchar *uri;
 	gchar *path;
-
-	/* list of uris (char *) that are to be excluded for this path only */
-	GSList *excluded;
 };
 typedef struct _BraseroGraftPt BraseroGraftPt;
 
@@ -191,7 +194,7 @@ brasero_track_set_data_file_num (BraseroTrack *track,
 
 BraseroBurnResult
 brasero_track_set_drive_source (BraseroTrack *track,
-				NautilusBurnDrive *drive);
+				BraseroDrive *drive);
 BraseroBurnResult
 brasero_track_set_drive_track (BraseroTrack *track,
 			       guint num);
@@ -218,7 +221,9 @@ brasero_track_get_audio_end (BraseroTrack *track);
 BraseroSongInfo *
 brasero_track_get_audio_info (BraseroTrack *track);
 
-NautilusBurnDrive *
+BraseroMedium *
+brasero_track_get_medium_source (BraseroTrack *track);
+BraseroDrive *
 brasero_track_get_drive_source (BraseroTrack *track);
 gint
 brasero_track_get_drive_track (BraseroTrack *track);
@@ -234,6 +239,7 @@ brasero_track_get_data_paths (BraseroTrack *track,
 			      const gchar *grafts_path,
 			      const gchar *excluded_path,
 			      const gchar *emptydir,
+			      const gchar *videodir,
 			      GError **error);
 
 gchar *
@@ -287,6 +293,20 @@ BraseroBurnResult
 brasero_track_get_data_file_num (BraseroTrack *track,
 				 gint64 *num_files);
 
+/**
+ *
+ */
+
+BraseroBurnResult
+brasero_track_tag_add (BraseroTrack *track,
+		       const gchar *tag,
+		       GValue *value);
+
+BraseroBurnResult
+brasero_track_tag_lookup (BraseroTrack *track,
+			  const gchar *tag,
+			  GValue **value);
+
 G_END_DECLS
 
 #endif /* _BURN_TRACK_H */
