@@ -35,10 +35,6 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
-#include <nautilus-burn-drive.h>
-
 #include "brasero-marshal.h"
 #include "burn-basics.h"
 #include "burn-debug.h"
@@ -49,12 +45,9 @@
 #include "burn-task-ctx.h"
 #include "burn-task.h"
 #include "burn-caps.h"
-#include "burn-volume.h"
-#include "brasero-ncb.h"
-
-#ifdef BUILD_DBUS
-  #include "burn-dbus.h"
-#endif
+#include "burn-volume-obj.h"
+#include "burn-drive.h"
+#include "burn-dbus.h"
 
 G_DEFINE_TYPE (BraseroBurn, brasero_burn, G_TYPE_OBJECT);
 
@@ -69,12 +62,10 @@ struct _BraseroBurnPrivate {
 	guint task_nb;
 	BraseroTask *task;
 
-	NautilusBurnDrive *src;
-	NautilusBurnDrive *dest;
+	BraseroDrive *src;
+	BraseroDrive *dest;
 
-#ifdef BUILD_DBUS
 	gint appcookie;
-#endif
 
 	guint src_locked:1;
 	guint dest_locked:1;
@@ -132,12 +123,13 @@ static guint brasero_burn_signals [LAST_SIGNAL] = { 0 };
 #define BRASERO_BURN_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_BURN, BraseroBurnPrivate))
 
 #define MAX_EJECT_WAIT_TIME	20000
-#define MAX_MOUNT_ATTEMPS	10
+#define MAX_MOUNT_ATTEMPTS	10
 #define MOUNT_TIMEOUT		500
 
-static GObjectClass *parent_class = NULL;
+#define MAX_LOAD_ATTEMPTS	20
+#define LOAD_TIMEOUT		500
 
-#ifdef BUILD_DBUS
+static GObjectClass *parent_class = NULL;
 
 static void
 brasero_burn_powermanagement (BraseroBurn *self,
@@ -151,8 +143,6 @@ brasero_burn_powermanagement (BraseroBurn *self,
 		brasero_uninhibit_suspend (priv->appcookie); 
 }
 
-#endif
-
 BraseroBurn *
 brasero_burn_new ()
 {
@@ -172,9 +162,7 @@ brasero_burn_log (BraseroBurn *burn,
 	va_list arg_list;
 
 	va_start (arg_list, format);
-
 	brasero_burn_session_logv (priv->session, format, arg_list);
-
 	va_end (arg_list);
 }
 
@@ -235,56 +223,26 @@ brasero_burn_sleep (BraseroBurn *burn, gint msec)
 	return BRASERO_BURN_CANCEL;
 }
 
-static gpointer
-_eject_async (gpointer data)
-{
-	NautilusBurnDrive *drive = NAUTILUS_BURN_DRIVE (data);
-
-	nautilus_burn_drive_eject (drive);
-	nautilus_burn_drive_unref (drive);
-
-	return NULL;
-}
-
-static void
-brasero_burn_eject_async (NautilusBurnDrive *drive)
-{
-	GError *error = NULL;
-
-	BRASERO_BURN_LOG ("Asynchronous ejection");
-	nautilus_burn_drive_ref (drive);
-	g_thread_create (_eject_async, drive, FALSE, &error);
-	if (error) {
-		g_warning ("Could not create thread %s\n", error->message);
-		g_error_free (error);
-
-		nautilus_burn_drive_unref (drive);
-		nautilus_burn_drive_eject (drive);
-	}
-}
-
 static BraseroBurnResult
-brasero_burn_eject_dest_media (BraseroBurn *self,
-			       GError **error)
+brasero_burn_wait_for_dest_insertion (BraseroBurn *burn,
+				      GError **error)
 {
-	BraseroBurnPrivate *priv;
-	guint elapsed = 0;
+	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+	BraseroMedium *medium;
+	guint attempt = 0;
+	gchar *failure;
 
-	priv = BRASERO_BURN_PRIVATE (self);
-
-	BRASERO_BURN_LOG ("Ejecting destination disc");
+	BRASERO_BURN_LOG ("Waiting for destination disc");
 	if (!priv->dest)
 		return BRASERO_BURN_OK;
 
-	if (nautilus_burn_drive_is_mounted (priv->dest))
-		nautilus_burn_drive_unmount (priv->dest);
-
+	/* we need to release our lock */
 	if (priv->dest_locked) {
 		priv->dest_locked = 0;
-		if (!nautilus_burn_drive_unlock (priv->dest)) {
+		if (!brasero_drive_unlock (priv->dest)) {
 			gchar *name;
 
-			name = nautilus_burn_drive_get_name_for_display (priv->dest);
+			name = brasero_drive_get_display_name (priv->dest);
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
@@ -295,21 +253,74 @@ brasero_burn_eject_dest_media (BraseroBurn *self,
 		}
 	}
 
-	brasero_burn_eject_async (priv->dest);
+	medium = brasero_drive_get_medium (priv->dest);
+	while (brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE) {
+		brasero_burn_sleep (burn, LOAD_TIMEOUT);
+		
+		attempt ++;
+		if (attempt > MAX_LOAD_ATTEMPTS) {
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("the disc could not be reloaded (max attemps reached)"));
+			return BRASERO_BURN_ERR;
+		}
 
-	/* sleep here to make sure that we got time to eject */
-	while (NCB_MEDIA_GET_STATUS (priv->dest) != BRASERO_MEDIUM_NONE) {
-		brasero_burn_sleep (self, 500);
-		elapsed += 500;
-
-		if (elapsed > MAX_EJECT_WAIT_TIME)
-			break;
+		medium = brasero_drive_get_medium (priv->dest);
 	}
 
-	if (NCB_MEDIA_GET_STATUS (priv->dest) != BRASERO_MEDIUM_NONE) {
+	/* Re-add the lock */
+	if (!priv->dest_locked
+	&&  !brasero_drive_lock (priv->dest, _("ongoing burning process"), &failure)) {
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     _("the drive can't be locked (%s)"),
+			     failure);
+		return BRASERO_BURN_ERR;
+	}
+	priv->dest_locked = 1;
+
+	return BRASERO_BURN_OK;
+}
+
+static BraseroBurnResult
+brasero_burn_eject_dest_media (BraseroBurn *self,
+			       GError **error)
+{
+	BraseroBurnPrivate *priv;
+	BraseroMedium *medium;
+
+	priv = BRASERO_BURN_PRIVATE (self);
+
+	BRASERO_BURN_LOG ("Ejecting destination disc");
+	if (!priv->dest)
+		return BRASERO_BURN_OK;
+
+	medium = brasero_drive_get_medium (priv->dest);
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium)))
+		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL);
+
+	if (priv->dest_locked) {
+		priv->dest_locked = 0;
+		if (!brasero_drive_unlock (priv->dest)) {
+			gchar *name;
+
+			name = brasero_drive_get_display_name (priv->dest);
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("\"%s\" can't be unlocked"),
+				     name);
+			g_free (name);
+			return BRASERO_BURN_ERR;
+		}
+	}
+
+	if (!brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, NULL)) {
 		gchar *name;
 
-		name = nautilus_burn_drive_get_name_for_display (priv->dest);
+		name = brasero_drive_get_display_name (priv->dest);
 
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
@@ -331,27 +342,28 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 			      GError **error)
 {
 	BraseroBurnPrivate *priv;
-	guint elapsed = 0;
+	BraseroMedium *medium;
 
 	priv = BRASERO_BURN_PRIVATE (self);
 
 	if (!priv->src)
 		return BRASERO_BURN_OK;
 
-	if (nautilus_burn_drive_is_mounted (priv->src)) {
+	medium = brasero_drive_get_medium (priv->src);
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
 		BraseroBurnResult result;
 
-		result = NCB_DRIVE_UNMOUNT (priv->src, error);
+		result = brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, error);
 		if (result != BRASERO_BURN_OK)
 			return result;
 	}
 
 	if (priv->src_locked) {
 		priv->src_locked = 0;
-		if (!nautilus_burn_drive_unlock (priv->src)) {
+		if (!brasero_drive_unlock (priv->src)) {
 			gchar *name;
 
-			name = nautilus_burn_drive_get_name_for_display (priv->src);
+			name = brasero_drive_get_display_name (priv->src);
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
@@ -362,21 +374,12 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 		}
 	}
 
-	brasero_burn_eject_async (priv->src);
-
-	/* sleep here to make sure that we got time to eject */
-	while (NCB_MEDIA_GET_STATUS (priv->src) != BRASERO_MEDIUM_NONE) {
-		brasero_burn_sleep (self, 500);
-		elapsed += 500;
-
-		if (elapsed > MAX_EJECT_WAIT_TIME)
-			break;
-	}
-
-	if (NCB_MEDIA_GET_STATUS (priv->src) != BRASERO_MEDIUM_NONE) {
+	brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, NULL);
+	medium = brasero_drive_get_medium (priv->src);
+	if (medium && brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
 		gchar *name;
 
-		name = nautilus_burn_drive_get_name_for_display (priv->src);
+		name = brasero_drive_get_display_name (priv->src);
 
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
@@ -396,7 +399,7 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 
 static BraseroBurnResult
 brasero_burn_ask_for_media (BraseroBurn *burn,
-			    NautilusBurnDrive *drive,
+			    BraseroDrive *drive,
 			    BraseroBurnError error_type,
 			    BraseroMedia required_media,
 			    GError **error)
@@ -442,9 +445,11 @@ brasero_burn_ask_for_src_media (BraseroBurn *burn,
 				GError **error)
 {
 	BraseroMedia media;
+	BraseroMedium *medium;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
-	media = NCB_MEDIA_GET_STATUS (priv->src);
+	medium = brasero_drive_get_medium (priv->src);
+	media = brasero_medium_get_status (medium);
 	if (media != BRASERO_MEDIUM_NONE) {
 		BraseroBurnResult result;
 		result = brasero_burn_eject_src_media (burn, error);
@@ -466,9 +471,11 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 				 GError **error)
 {
 	BraseroMedia media;
+	BraseroMedium *medium;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
-	media = NCB_MEDIA_GET_STATUS (priv->dest);
+	medium = brasero_drive_get_medium (priv->dest);
+	media = brasero_medium_get_status (medium);
 	if (media != BRASERO_MEDIUM_NONE) {
 		BraseroBurnResult result;
 
@@ -477,6 +484,17 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 			return result;
 	}
 
+	if (!priv->dest) {
+		priv->dest = brasero_burn_session_get_burner (priv->session);
+		if (!priv->dest) {
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("no drive specified"));
+			return BRASERO_BURN_ERR;
+		}
+	}
+
 	return brasero_burn_ask_for_media (burn,
 					   priv->dest,
 					   error_type,
@@ -490,6 +508,7 @@ brasero_burn_lock_src_media (BraseroBurn *burn,
 {
 	gchar *failure;
 	BraseroMedia media;
+	BraseroMedium *medium;
 	BraseroBurnResult result;
 	BraseroBurnError error_type;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
@@ -505,15 +524,17 @@ brasero_burn_lock_src_media (BraseroBurn *burn,
 
 
 again:
-	if (nautilus_burn_drive_is_mounted (priv->src)) {
-		if (!NCB_DRIVE_UNMOUNT (priv->src, NULL))
+
+	medium = brasero_drive_get_medium (priv->src);
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
+		if (!brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL))
 			g_warning ("Couldn't unmount volume in drive: %s",
-				   NCB_DRIVE_GET_DEVICE (priv->src));
+				   brasero_drive_get_device (priv->src));
 	}
 
 	/* NOTE: we used to unmount the media before now we shouldn't need that
 	 * get any information from the drive */
-	media = NCB_MEDIA_GET_STATUS (priv->src);
+	media = brasero_medium_get_status (medium);
 	if (media == BRASERO_MEDIUM_NONE)
 		error_type = BRASERO_BURN_ERROR_MEDIA_NONE;
 	else if (media == BRASERO_MEDIUM_BUSY)
@@ -537,7 +558,7 @@ again:
 	}
 
 	if (!priv->src_locked
-	&&  !nautilus_burn_drive_lock (priv->src, _("ongoing copying process"), &failure)) {
+	&&  !brasero_drive_lock (priv->src, _("ongoing copying process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -571,10 +592,11 @@ brasero_burn_reload_src_media (BraseroBurn *burn,
 
 static BraseroBurnResult
 brasero_burn_lock_rewritable_media (BraseroBurn *burn,
-					GError **error)
+				    GError **error)
 {
 	gchar *failure;
 	BraseroMedia media;
+	BraseroMedium *medium;
 	BraseroBurnResult result;
 	BraseroBurnError error_type;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
@@ -588,7 +610,10 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
-	if (!nautilus_burn_drive_can_rewrite (priv->dest)) {
+ again:
+
+	medium = brasero_drive_get_medium (priv->dest);
+	if (!brasero_medium_can_be_rewritten (medium)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -596,15 +621,13 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
- again:
-
-	if (nautilus_burn_drive_is_mounted (priv->dest)) {
-		if (!NCB_DRIVE_UNMOUNT (priv->dest, NULL))
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
+		if (!brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL))
 			g_warning ("Couldn't unmount volume in drive: %s",
-				   NCB_DRIVE_GET_DEVICE (priv->dest));
+				   brasero_drive_get_device (priv->dest));
 	}
 
-	media = NCB_MEDIA_GET_STATUS (priv->dest);
+	media = brasero_medium_get_status (medium);
 	if (media == BRASERO_MEDIUM_NONE)
 		error_type = BRASERO_BURN_ERROR_MEDIA_NONE;
 	else if (media == BRASERO_MEDIUM_BUSY)
@@ -630,7 +653,7 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 	}
 
 	if (!priv->dest_locked
-	&&  !nautilus_burn_drive_lock (priv->dest, _("ongoing blanking process"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("ongoing blanking process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -707,7 +730,7 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 
 		return BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE;
 	}
-	else if (unsupported & BRASERO_MEDIUM_DVD)
+	else if (unsupported & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
 		return BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED;
 
 	return BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
@@ -721,6 +744,7 @@ brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
 	gboolean must_blank;
 	BraseroBurnFlag flags;
 	BraseroTrackType input;
+	BraseroMedium *medium;
 	BraseroBurnError berror;
 	BraseroBurnResult result;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
@@ -736,27 +760,36 @@ brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
 
 	brasero_burn_session_get_input_type (priv->session, &input);
 	flags = brasero_burn_session_get_flags (priv->session);
-	if (!nautilus_burn_drive_can_write (priv->dest)) {
+
+	result = BRASERO_BURN_OK;
+
+again:
+
+	medium = brasero_drive_get_medium (priv->dest);
+	if (!medium) {
+		result = BRASERO_BURN_NEED_RELOAD;
+		berror = BRASERO_BURN_ERROR_MEDIA_NONE;
+		goto end;
+	}
+
+	if (!brasero_medium_can_be_written (medium)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("the drive has no burning capabilities"));
 		BRASERO_BURN_NOT_SUPPORTED_LOG (burn);
 	}
-	result = BRASERO_BURN_OK;
-
-again:
 
 	/* if drive is mounted then unmount before checking anything */
-	if (nautilus_burn_drive_is_mounted (priv->dest)) {
-		if (!NCB_DRIVE_UNMOUNT (priv->dest, NULL))
-			g_warning ("Couldn't unmount volume in drive: %s",
-				   NCB_DRIVE_GET_DEVICE (priv->dest));
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
+		if (!brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL))
+			BRASERO_BURN_LOG ("Couldn't unmount volume in drive: %s",
+					  brasero_drive_get_device (priv->dest));
 	}
 
 	berror = BRASERO_BURN_ERROR_NONE;
-	media = NCB_MEDIA_GET_STATUS (priv->dest);
 
+	media = brasero_medium_get_status (medium);
 	BRASERO_BURN_LOG_WITH_FULL_TYPE (BRASERO_TRACK_TYPE_DISC,
 					 media,
 					 BRASERO_PLUGIN_IO_NONE,
@@ -860,7 +893,7 @@ again:
 	}
 
 	if (!priv->dest_locked
-	&&  !nautilus_burn_drive_lock (priv->dest, _("ongoing burning process"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("ongoing burning process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -889,7 +922,7 @@ end:
 
 	if (result != BRASERO_BURN_OK) {
 		priv->dest_locked = 0;
-		nautilus_burn_drive_unlock (priv->dest);
+		brasero_drive_unlock (priv->dest);
 	}
 
 	return result;
@@ -907,9 +940,8 @@ brasero_burn_reload_dest_media (BraseroBurn *burn,
 again:
 
 	/* eject and ask the user to reload a disc */
-	required_media = brasero_burn_caps_get_required_media_type (priv->caps,
-								    priv->session);
-	required_media &= (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_CD|BRASERO_MEDIUM_DVD);
+	required_media = brasero_burn_caps_get_required_media_type (priv->caps, priv->session);
+	required_media &= (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_CD|BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL);
 
 	result = brasero_burn_ask_for_dest_media (burn,
 						  error_code,
@@ -930,6 +962,7 @@ brasero_burn_mount_media (BraseroBurn *self,
 			  GError **error)
 {
 	guint retries = 0;
+	BraseroMedium *medium;
 	BraseroBurnPrivate *priv;
 
 	priv = BRASERO_BURN_PRIVATE (self);
@@ -940,8 +973,9 @@ brasero_burn_mount_media (BraseroBurn *self,
 		       0,
 		       BRASERO_BURN_ACTION_CHECKSUM);
 
-	while (!nautilus_burn_drive_is_mounted (priv->dest)) {
-		if (retries++ > MAX_MOUNT_ATTEMPS) {
+	medium = brasero_drive_get_medium (priv->dest);
+	while (!brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
+		if (retries++ > MAX_MOUNT_ATTEMPTS) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
@@ -950,7 +984,7 @@ brasero_burn_mount_media (BraseroBurn *self,
 		}
 
 		/* NOTE: we don't really care about the return value */
-		NCB_DRIVE_MOUNT (priv->dest, error);
+		brasero_volume_mount (BRASERO_VOLUME (medium), FALSE, NULL);
 		priv->mounted_by_us = TRUE;
 
 		brasero_burn_sleep (self, MOUNT_TIMEOUT);
@@ -965,6 +999,7 @@ brasero_burn_lock_checksum_media (BraseroBurn *burn,
 {
 	gchar *failure;
 	BraseroMedia media;
+	BraseroMedium *medium;
 	BraseroBurnResult result;
 	BraseroBurnError error_type;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
@@ -973,7 +1008,8 @@ brasero_burn_lock_checksum_media (BraseroBurn *burn,
 
 again:
 
-	media = NCB_MEDIA_GET_STATUS (priv->dest);
+	medium = brasero_drive_get_medium (priv->dest);
+	media = brasero_medium_get_status (medium);
 	error_type = BRASERO_BURN_ERROR_NONE;
 	BRASERO_BURN_LOG_DISC_TYPE (media, "Waiting for media to checksum");
 
@@ -984,7 +1020,6 @@ again:
 							  BRASERO_BURN_WARNING_CHECKSUM,
 							  BRASERO_MEDIUM_NONE,
 							  error);
-
 		if (result != BRASERO_BURN_OK)
 			return result;
 	}
@@ -1007,7 +1042,7 @@ again:
 	}
 
 	if (!priv->dest_locked
-	&&  !nautilus_burn_drive_lock (priv->dest, _("ongoing checksuming operation"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("ongoing checksuming operation"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -1017,11 +1052,11 @@ again:
 	}
 
 	/* if drive is mounted then unmount before checking anything */
-	if (nautilus_burn_drive_is_mounted (priv->dest)
-	&& !NCB_DRIVE_UNMOUNT (priv->dest, NULL))
+/*	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))
+	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL))
 		g_warning ("Couldn't unmount volume in drive: %s",
-			   NCB_DRIVE_GET_DEVICE (priv->dest));
-
+			   brasero_drive_get_device (priv->dest));
+*/
 	priv->dest_locked = 1;
 
 	return BRASERO_BURN_OK;
@@ -1031,6 +1066,7 @@ static BraseroBurnResult
 brasero_burn_unlock_src_media (BraseroBurn *burn)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+	BraseroMedium *medium;
 
 	if (!priv->src)
 		return BRASERO_BURN_OK;
@@ -1040,16 +1076,17 @@ brasero_burn_unlock_src_media (BraseroBurn *burn)
 		return BRASERO_BURN_OK;
 	}
 
+	medium = brasero_drive_get_medium (priv->src);
 	if (priv->mounted_by_us) {
-		nautilus_burn_drive_unmount (priv->src);
+		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL);
 		priv->mounted_by_us = 0;
 	}
 
 	priv->src_locked = 0;
-	nautilus_burn_drive_unlock (priv->src);
+	brasero_drive_unlock (priv->src);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_burn_eject_async (priv->src);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
 
 	priv->src = NULL;
 	return BRASERO_BURN_OK;
@@ -1059,6 +1096,7 @@ static BraseroBurnResult
 brasero_burn_unlock_dest_media (BraseroBurn *burn)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+	BraseroMedium *medium;
 
 	if (!priv->dest)
 		return BRASERO_BURN_OK;
@@ -1069,10 +1107,11 @@ brasero_burn_unlock_dest_media (BraseroBurn *burn)
 	}
 
 	priv->dest_locked = 0;
-	nautilus_burn_drive_unlock (priv->dest);
+	brasero_drive_unlock (priv->dest);
+	medium = brasero_drive_get_medium (priv->dest);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_burn_eject_async (priv->dest);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
 
 	priv->dest = NULL;
 	return BRASERO_BURN_OK;
@@ -1227,14 +1266,16 @@ brasero_burn_ask_for_joliet (BraseroBurn *burn)
 static BraseroBurnResult
 brasero_burn_run_eraser (BraseroBurn *burn, GError **error)
 {
-	NautilusBurnDrive *drive;
+	BraseroDrive *drive;
+	BraseroMedium *medium;
 	BraseroBurnPrivate *priv;
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
 	drive = brasero_burn_session_get_burner (priv->session);
-	if (nautilus_burn_drive_is_mounted (drive)
-	&& !NCB_DRIVE_UNMOUNT (drive, NULL)) {
+	medium = brasero_drive_get_medium (drive);
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))
+	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_BUSY_DRIVE,
@@ -1254,16 +1295,19 @@ brasero_burn_run_imager (BraseroBurn *burn,
 	BraseroBurnError error_code;
 	BraseroBurnResult result;
 	GError *ret_error = NULL;
-	NautilusBurnDrive *src;
+	BraseroMedium *medium;
+	BraseroDrive *src;
 
 	src = brasero_burn_session_get_src_drive (priv->session);
 
 start:
 
+	medium = brasero_drive_get_medium (src);
+
 	/* this is just in case */
-	if (src
-	&&  nautilus_burn_drive_is_mounted (src)
-	&& !NCB_DRIVE_UNMOUNT (src, NULL)) {
+	if (medium
+	&&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
+	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_BUSY_DRIVE,
@@ -1292,8 +1336,9 @@ start:
 	}
 
 	if (result != BRASERO_BURN_ERR) {
-		if (error)
+		if (error && ret_error)
 			g_propagate_error (error, ret_error);
+
 		return result;
 	}
 
@@ -1337,7 +1382,9 @@ start:
 	 */
 
 	/* not recoverable propagate the error */
-	g_propagate_error (error, ret_error);
+	if (error && ret_error)
+		g_propagate_error (error, ret_error);
+
 	return BRASERO_BURN_ERR;
 }
 
@@ -1345,36 +1392,42 @@ static BraseroBurnResult
 brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 {
 	gint error_code;
+	BraseroDrive *src;
 	gboolean has_slept;
-	NautilusBurnDrive *src;
+	BraseroDrive *burner;
 	GError *ret_error = NULL;
 	BraseroBurnResult result;
-	NautilusBurnDrive *burner;
+	BraseroMedium *src_medium;
+	BraseroMedium *burnt_medium;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
 	has_slept = FALSE;
 	src = brasero_burn_session_get_src_drive (priv->session);
+	src_medium = brasero_drive_get_medium (src);
 	burner = brasero_burn_session_get_burner (priv->session);
+	burnt_medium = brasero_drive_get_medium (burner);
 
 start:
 
 	/* this is just in case */
 	if (BRASERO_BURN_SESSION_NO_TMP_FILE (priv->session)
-	&&  src
-	&&  nautilus_burn_drive_is_mounted (src)
-	&& !NCB_DRIVE_UNMOUNT (src, NULL)) {
+	&&  src_medium
+	&&  brasero_volume_is_mounted (BRASERO_VOLUME (src_medium))
+	&& !brasero_volume_umount (BRASERO_VOLUME (src_medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_BUSY_DRIVE,
 			     _("the drive seems to be busy"));
 		return BRASERO_BURN_ERR;
 	}
-	else if (nautilus_burn_drive_is_mounted (burner)
-	     && !NCB_DRIVE_UNMOUNT (burner, NULL)) {
-		ret_error = g_error_new (BRASERO_BURN_ERROR,
-					 BRASERO_BURN_ERROR_BUSY_DRIVE,
-					 _("the drive seems to be busy"));
-		result = BRASERO_BURN_ERR;
+
+	if (brasero_volume_is_mounted (BRASERO_VOLUME (burnt_medium))
+	&& !brasero_volume_umount (BRASERO_VOLUME (burnt_medium), TRUE, NULL)) {
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_BUSY_DRIVE,
+			     _("the drive seems to be busy"));
+		return BRASERO_BURN_ERR;
 	}
 
 	/* actual running of task */
@@ -1410,7 +1463,9 @@ start:
 		 * if the user wants to carry on with a non joliet disc */
 		result = brasero_burn_ask_for_joliet (burn);
 		if (result != BRASERO_BURN_OK) {
-			g_propagate_error (error, ret_error);
+			if (ret_error)
+				g_propagate_error (error, ret_error);
+
 			return result;
 		}
 
@@ -1515,7 +1570,9 @@ start:
 		goto start;
 	}
 
-	g_propagate_error (error, ret_error);
+	if (ret_error)
+		g_propagate_error (error, ret_error);
+
 	return BRASERO_BURN_ERR;
 }
 
@@ -1559,7 +1616,7 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 		/* see what type of task it is. It could be a blank/erase one */
 		action = brasero_task_ctx_get_action (BRASERO_TASK_CTX (priv->task));
 		if (action == BRASERO_TASK_ACTION_ERASE) {
-			/* this is to avoid a potential problem when running a 
+			/* This is to avoid a potential problem when running a 
 			 * dummy session first. When running dummy session the 
 			 * media gets erased if need be. Since it is not
 			 * reloaded afterwards, for brasero it has still got 
@@ -1575,6 +1632,14 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 			g_object_unref (priv->task);
 			priv->task = NULL;
 			priv->tasks_done ++;
+
+			/* Now it can happen (like with dvd+rw-format) that for
+			 * the whole OS, the disc doesn't exist during the 
+			 * formatting. Wait for the disc to reappear */
+			result = brasero_burn_wait_for_dest_insertion (burn, error);
+			if (result != BRASERO_BURN_OK)
+				break;
+
 			continue;
 		}
 
@@ -1625,6 +1690,7 @@ brasero_burn_check_real (BraseroBurn *self,
 			 BraseroTrack *track,
 			 GError **error)
 {
+	BraseroMedium *medium;
 	BraseroTrackType type;
 	BraseroBurnResult result;
 	BraseroBurnPrivate *priv;
@@ -1638,9 +1704,12 @@ brasero_burn_check_real (BraseroBurn *self,
 	brasero_track_get_type (track, &type);
 
 	/* if the input is a DISC, ask/mount/unmount and lock it (as dest) */
+	medium = brasero_drive_get_medium (priv->dest);
 	if (type.type == BRASERO_TRACK_TYPE_DISC
-	&&  checksum_type == BRASERO_CHECKSUM_MD5_FILE
-	&& !nautilus_burn_drive_is_mounted (priv->dest)) {
+	&& (checksum_type == BRASERO_CHECKSUM_MD5_FILE
+	||  checksum_type == BRASERO_CHECKSUM_SHA1_FILE
+	||  checksum_type == BRASERO_CHECKSUM_SHA256_FILE)
+	&& !brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
 		result = brasero_burn_mount_media (self, error);
 		if (result != BRASERO_BURN_OK)
 			return result;
@@ -1666,9 +1735,11 @@ brasero_burn_check_real (BraseroBurn *self,
 		/* make sure one last time it is not mounted IF and only IF the
 		 * checksum type is NOT FILE_MD5 */
 		if (priv->dest
-		&&  checksum_type == BRASERO_CHECKSUM_MD5
-		&&  nautilus_burn_drive_is_mounted (priv->dest)
-		&& !NCB_DRIVE_UNMOUNT (priv->dest, NULL)) {
+		&& (checksum_type == BRASERO_CHECKSUM_MD5
+		||  checksum_type == BRASERO_CHECKSUM_SHA1
+		||  checksum_type == BRASERO_CHECKSUM_SHA256)
+		&&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
+		&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_BUSY_DRIVE,
@@ -1721,13 +1792,13 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("there is no track to be burn"));
+			     _("there is no track to be burnt"));
 		return BRASERO_BURN_ERR;
 	}
 
 	/* make sure there is a drive set as burner */
 	if (!brasero_burn_session_is_dest_file (priv->session)) {
-		NautilusBurnDrive *burner;
+		BraseroDrive *burner;
 
 		burner = brasero_burn_session_get_burner (priv->session);
 		if (!burner) {
@@ -1763,6 +1834,8 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 						     &supported,
 						     &compulsory);
 		}
+		else if (flags & flag)
+			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported");
 	}
 
 	retval = brasero_burn_session_get_flags (priv->session);
@@ -1854,6 +1927,7 @@ brasero_burn_record_session (BraseroBurn *burn,
 	BraseroBurnPrivate *priv;
 	BraseroBurnResult result;
 	GError *ret_error = NULL;
+	BraseroMedium *medium;
 	GSList *tracks;
 
 	priv = BRASERO_BURN_PRIVATE (burn);
@@ -1955,13 +2029,16 @@ brasero_burn_record_session (BraseroBurn *burn,
 	 * unlock/eject step. A better way would be to have a system call to 
 	 * force a re-load. */
 
-	result = brasero_burn_eject_dest_media (burn, error);
+/*	result = brasero_burn_eject_dest_media (burn, error);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
 	priv->dest = NULL;
+*/
 
-	if (type == BRASERO_CHECKSUM_MD5) {
+	if (type == BRASERO_CHECKSUM_MD5
+	||  type == BRASERO_CHECKSUM_SHA1
+	||  type == BRASERO_CHECKSUM_SHA256) {
 		const gchar *checksum = NULL;
 
 		checksum = brasero_track_get_checksum (track);
@@ -1975,9 +2052,21 @@ brasero_burn_record_session (BraseroBurn *burn,
 	else if (type == BRASERO_CHECKSUM_MD5_FILE) {
 		track = brasero_track_new (BRASERO_TRACK_TYPE_DISC);
 		brasero_track_set_checksum (track,
-					    BRASERO_CHECKSUM_MD5_FILE,
+					    type,
 					    BRASERO_MD5_FILE);
 	}
+	else if (type == BRASERO_CHECKSUM_SHA1_FILE) {
+		track = brasero_track_new (BRASERO_TRACK_TYPE_DISC);
+		brasero_track_set_checksum (track,
+					    type,
+					    BRASERO_SHA1_FILE);
+	}
+	else if (type == BRASERO_CHECKSUM_SHA256_FILE) {
+		track = brasero_track_new (BRASERO_TRACK_TYPE_DISC);
+		brasero_track_set_checksum (track,
+					    type,
+					    BRASERO_SHA256_FILE);
+	}
 
 	brasero_burn_session_push_tracks (priv->session);
 
@@ -1989,21 +2078,31 @@ brasero_burn_record_session (BraseroBurn *burn,
 	brasero_track_unref (track);
 
 	/* reload media */
-	result = brasero_burn_lock_checksum_media (burn, error);
+/*	result = brasero_burn_lock_checksum_media (burn, error);
 	if (result != BRASERO_BURN_OK)
 		return result;
-
-	/* this may be necessary for the drive to settle down and possibly be 
-	 * mounted by gnome-volume-manager (just temporarily) */
+*/
+	/* this may be necessary for the drive to settle down */
+	/* and possibly be mounted by gnome-volume-manager (just temporarily) */
 	result = brasero_burn_sleep (burn, 5000);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	if (type == BRASERO_CHECKSUM_MD5) {
+	medium = brasero_drive_get_medium (priv->dest);
+	brasero_medium_reload_info (medium);
+
+	if (type == BRASERO_CHECKSUM_MD5
+	||  type == BRASERO_CHECKSUM_SHA1
+	||  type == BRASERO_CHECKSUM_SHA256) {
 		guint track_num;
+		BraseroDrive *drive;
+		BraseroMedium *medium;
 
 		/* get the last track number */
-		track_num = NCB_MEDIA_GET_TRACK_NUM (brasero_burn_session_get_burner (priv->session));
+		drive = brasero_burn_session_get_burner (priv->session);
+		medium = brasero_drive_get_medium (drive);
+		track_num = brasero_medium_get_track_num (medium);
+
 		BRASERO_BURN_LOG ("Last written track num == %i", track_num);
 		brasero_track_set_drive_track (track, track_num);
 	}
@@ -2061,18 +2160,13 @@ brasero_burn_check (BraseroBurn *self,
 			return result;
 	}
 
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (self, TRUE);
-#endif
 
 	result = brasero_burn_check_real (self, track, error);
 
 	brasero_burn_unlock_medias (self);
 
-
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (self, FALSE);
-#endif
 
 	/* no need to check the result of the comparison, it's set in session */
 	priv->session = NULL;
@@ -2118,7 +2212,7 @@ brasero_burn_same_src_dest (BraseroBurn *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	/* get a new output. Also ask for both  */
+	/* get a new output. Also ask for both */
 	brasero_burn_session_push_settings (priv->session);
 	result = brasero_burn_session_get_tmp_image (priv->session,
 						     format,
@@ -2189,9 +2283,7 @@ brasero_burn_record (BraseroBurn *burn,
 	g_object_ref (session);
 	priv->session = session;
 
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (burn, TRUE);
-#endif
 
 	/* say to the whole world we started */
 	brasero_burn_action_changed_real (burn, BRASERO_BURN_ACTION_PREPARING);
@@ -2201,11 +2293,18 @@ brasero_burn_record (BraseroBurn *burn,
 		result = brasero_burn_same_src_dest (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
-	}
 
-	/* do some drive locking quite early to make sure we have a media
-	 * in the drive so that we'll have all the necessary information */
-	if (!brasero_burn_session_is_dest_file (session)) {
+		/* lock the dest drive do it this way for the message */
+		result = brasero_burn_reload_dest_media (burn,
+							 BRASERO_BURN_WARNING_INSERT_AFTER_COPY,
+							 error);
+		if (result != BRASERO_BURN_OK)
+			goto end;
+	}
+	else if (!brasero_burn_session_is_dest_file (session)) {
+		/* do some drive locking quite early to make sure we have a
+		 * media in the drive so that we'll have all the necessary
+		 * information */
 		result = brasero_burn_lock_dest_media (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
@@ -2275,9 +2374,7 @@ end:
 	else
 		BRASERO_BURN_DEBUG (burn, "Session successfully finished");
 
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (burn, FALSE);
-#endif
 
 	/* release session */
 	g_object_unref (priv->session);
@@ -2336,9 +2433,7 @@ brasero_burn_blank (BraseroBurn *burn,
 	g_object_ref (session);
 	priv->session = session;
 
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (burn, TRUE);
-#endif
 
 	/* we wait for the insertion of a media and lock it */
 	result = brasero_burn_lock_rewritable_media (burn, error);
@@ -2377,9 +2472,7 @@ end:
 		brasero_burn_action_changed_real (burn, BRASERO_BURN_ACTION_FINISHED);
 
 
-#ifdef BUILD_DBUS
 	brasero_burn_powermanagement (burn, FALSE);
-#endif
 
 	/* release session */
 	g_object_unref (priv->session);
@@ -2500,7 +2593,7 @@ brasero_burn_class_init (BraseroBurnClass *klass)
 			      brasero_marshal_INT__OBJECT_INT_INT,
 			      G_TYPE_INT, 
 			      3,
-			      NAUTILUS_BURN_TYPE_DRIVE,
+			      BRASERO_TYPE_DRIVE,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
         brasero_burn_signals [PROGRESS_CHANGED_SIGNAL] =
