@@ -43,8 +43,6 @@
 
 #include <gconf/gconf-client.h>
 
-#include <nautilus-burn-drive.h>
-
 #include "burn-basics.h"
 #include "burn-job.h"
 #include "burn-process.h"
@@ -149,7 +147,8 @@ brasero_cdrecord_stderr_read (BraseroProcess *process, const gchar *line)
 						BRASERO_BURN_ERROR_SLOW_DMA,
 						_("The system is too slow to write the CD at this speed. Try a lower speed")));
 	}
-	else if (strstr (line, "Operation not permitted. Cannot send SCSI cmd via ioctl")) {
+	else if (strstr (line, "Operation not permitted. Cannot send SCSI cmd via ioctl")
+	     ||  strstr (line, "Cannot open or use SCSI driver")) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_SCSI_IOCTL,
@@ -303,6 +302,12 @@ brasero_cdrecord_stdout_read (BraseroProcess *process, const gchar *line)
 /*		if (mb_total > 0)
 			priv->tracks_total_bytes += mb_total * 1048576;
 */	}
+	else if (strstr (line, "Formatting media")) {
+		brasero_job_set_current_action (BRASERO_JOB (process),
+						BRASERO_BURN_ACTION_BLANKING,
+						_("Formatting media"),
+						FALSE);
+	}
 	else if (strstr (line, "Sending CUE sheet")) {
 		BraseroTrackType type;
 
@@ -820,8 +825,10 @@ brasero_cdrecord_set_argv_record (BraseroCDRecord *cdrecord,
 			if (!rawpath)
 				BRASERO_JOB_NOT_READY (cdrecord);
 
-			if (flags & BRASERO_BURN_FLAG_DAO)
-				return BRASERO_BURN_ERR;
+			/* NOTE: we ignore DAO flag on purpose since it isn't
+			 * implemented yet. Don't error out since there is no
+			 * way for us to tell that we don't support this flag
+			 * for this specific input. */
 
 			g_ptr_array_add (argv, g_strdup ("fs=16m"));
 			g_ptr_array_add (argv, g_strdup ("-raw96r"));
@@ -899,7 +906,13 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 	g_ptr_array_add (argv, g_strdup ("cdrecord"));
 	g_ptr_array_add (argv, g_strdup ("-v"));
 
+#ifdef HAVE_CAM_LIB_H
+	/* FreeBSD like that better */
+	brasero_job_get_bus_target_lun (BRASERO_JOB (cdrecord), &device);
+#else
 	brasero_job_get_device (BRASERO_JOB (cdrecord), &device);
+#endif
+
 	dev_str = g_strdup_printf ("dev=%s", device);
 	g_ptr_array_add (argv, dev_str);
 	g_free (device);
@@ -1014,6 +1027,15 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 				   BRASERO_MEDIUM_APPENDABLE|
 				   BRASERO_MEDIUM_HAS_AUDIO|
 				   BRASERO_MEDIUM_HAS_DATA;
+	const BraseroMedia dvd_media = BRASERO_MEDIUM_DVD|
+				       BRASERO_MEDIUM_PLUS|
+				       BRASERO_MEDIUM_SEQUENTIAL|
+				       BRASERO_MEDIUM_WRITABLE|
+				       BRASERO_MEDIUM_REWRITABLE|
+				       BRASERO_MEDIUM_BLANK|
+				       BRASERO_MEDIUM_UNFORMATTED|
+				       BRASERO_MEDIUM_APPENDABLE|
+				       BRASERO_MEDIUM_HAS_DATA;
 	const BraseroMedia media_rw = BRASERO_MEDIUM_CD|
 				      BRASERO_MEDIUM_REWRITABLE|
 				      BRASERO_MEDIUM_APPENDABLE|
@@ -1021,7 +1043,7 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 				      BRASERO_MEDIUM_HAS_AUDIO|
 				      BRASERO_MEDIUM_HAS_DATA|
 				      BRASERO_MEDIUM_BLANK;
-	gchar *prog_name;
+	BraseroBurnResult result;
 	GSList *output;
 	GSList *input;
 
@@ -1030,23 +1052,24 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 			       "cdrecord",
 			       _("use cdrecord to burn CDs"),
 			       "Philippe Rouquier",
-			       0);
+			       1);
 
-	/* First see if this plugin can be used, i.e. if cdrecord is in
-	 * the path */
-	prog_name = g_find_program_in_path ("cdrecord");
-	if (!prog_name) {
-		*error = g_strdup (_("cdrecord could not be found in the path"));
-		return BRASERO_BURN_ERR;
-	}
-	g_free (prog_name);
+	/* First see if this plugin can be used */
+	result = brasero_process_check_path ("cdrecord", error);
+	if (result != BRASERO_BURN_OK)
+		return result;
 
 	/* for recording */
-	output = brasero_caps_disc_new (media);
 	input = brasero_caps_image_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
 					BRASERO_PLUGIN_IO_ACCEPT_FILE,
 					BRASERO_IMAGE_FORMAT_BIN);
 
+	/* cdrecord can burn all DVDs (except restricted) when it's ISOs */
+	output = brasero_caps_disc_new (dvd_media);
+	brasero_plugin_link_caps (plugin, output, input);
+	g_slist_free (output);
+
+	output = brasero_caps_disc_new (media);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (input);
 
@@ -1065,6 +1088,65 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 	g_slist_free (input);
 
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_DVD|
+				  BRASERO_MEDIUM_SEQUENTIAL|
+				  BRASERO_MEDIUM_WRITABLE|
+				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_BLANK,
+				  BRASERO_BURN_FLAG_DAO|
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_DUMMY|
+				  BRASERO_BURN_FLAG_NOGRACE,
+				  BRASERO_BURN_FLAG_NONE);
+
+	/* DVD+ R/RW don't support dummy mode 
+	 * NOTE: don't mix dao and multisession */
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_DVDR_PLUS|
+				  BRASERO_MEDIUM_BLANK,
+				  BRASERO_BURN_FLAG_DAO|
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_NOGRACE,
+				  BRASERO_BURN_FLAG_NONE);
+
+	/* for DVD+RW */
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_DVDRW_PLUS|
+				  BRASERO_MEDIUM_UNFORMATTED|
+				  BRASERO_MEDIUM_BLANK,
+				  BRASERO_BURN_FLAG_DAO|
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_NOGRACE,
+				  BRASERO_BURN_FLAG_NONE);
+
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_DVDRW_PLUS|
+				  BRASERO_MEDIUM_APPENDABLE|
+				  BRASERO_MEDIUM_CLOSED|
+				  BRASERO_MEDIUM_HAS_DATA,
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_NOGRACE|
+				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
+				  BRASERO_BURN_FLAG_NONE);
+
+	/* Flags for CD (RW)s */
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_CD|
+				  BRASERO_MEDIUM_WRITABLE|
+				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_BLANK,
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_MULTI|
+				  BRASERO_BURN_FLAG_DUMMY|
+				  BRASERO_BURN_FLAG_NOGRACE,
+				  BRASERO_BURN_FLAG_NONE);
+
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_CD|
 				  BRASERO_MEDIUM_WRITABLE|
@@ -1073,7 +1155,6 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_DAO|
 				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
 				  BRASERO_BURN_FLAG_DUMMY|
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
@@ -1095,7 +1176,33 @@ brasero_cdrecord_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
 
-	/* for blanking */
+	/* blanking/formatting caps and flags for +/sequential RW
+	 * NOTE: restricted overwrite DVD-RW can't be formatted.
+	 * moreover DVD+RW are formatted while DVD-RW sequential are blanked.
+	  * NOTE: blanking DVD-RW doesn't work */
+	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
+					BRASERO_MEDIUM_PLUS|
+					BRASERO_MEDIUM_REWRITABLE|
+					BRASERO_MEDIUM_APPENDABLE|
+					BRASERO_MEDIUM_CLOSED|
+					BRASERO_MEDIUM_HAS_DATA|
+					BRASERO_MEDIUM_UNFORMATTED|
+					BRASERO_MEDIUM_BLANK);
+	brasero_plugin_blank_caps (plugin, output);
+	g_slist_free (output);
+
+	/* again DVD+RW don't support dummy */
+	brasero_plugin_set_blank_flags (plugin,
+					BRASERO_MEDIUM_DVDRW_PLUS|
+					BRASERO_MEDIUM_APPENDABLE|
+					BRASERO_MEDIUM_HAS_DATA|
+					BRASERO_MEDIUM_UNFORMATTED|
+					BRASERO_MEDIUM_BLANK|
+					BRASERO_MEDIUM_CLOSED,
+					BRASERO_BURN_FLAG_NOGRACE,
+					BRASERO_BURN_FLAG_NONE);
+
+	/* for blanking (CDRWs) */
 	output = brasero_caps_disc_new (media_rw);
 	brasero_plugin_blank_caps (plugin, output);
 	g_slist_free (output);
