@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <glib.h>
 #include <glib-object.h>
@@ -37,8 +38,6 @@
 #include <glib/gi18n-lib.h>
 #include <gmodule.h>
 
-#include <nautilus-burn-drive.h>
-
 #include "burn-basics.h"
 #include "burn-debug.h"
 #include "burn-job.h"
@@ -46,7 +45,7 @@
 #include "burn-dvdcss.h"
 #include "burn-dvdcss-private.h"
 #include "burn-volume.h"
-#include "brasero-ncb.h"
+#include "burn-medium.h"
 
 BRASERO_PLUGIN_BOILERPLATE (BraseroDvdcss, brasero_dvdcss, BRASERO_TYPE_JOB, BraseroJob);
 
@@ -248,20 +247,31 @@ brasero_dvdcss_create_scrambled_sectors_map (GQueue *map,
 		if (!file->isdir) {
 			if (!strncmp (file->name + strlen (file->name) - 6, ".VOB", 4)) {
 				BraseroScrambledSectorRange *range;
+				GSList *extents;
 
 				range = g_new0 (BraseroScrambledSectorRange, 1);
-				range->start = file->specific.file.address_block;
-				range->end = range->start + (file->specific.file.size_bytes / DVDCSS_BLOCK_SIZE);
 
-				g_queue_push_head (map, range);
-
-				if (dvdcss_seek (handle, range->start, DVDCSS_SEEK_KEY) != range->start) {
-					g_set_error (error,
-						     BRASERO_BURN_ERROR,
-						     BRASERO_BURN_ERROR_GENERAL,
-						     _("Error reading video DVD (%s)"),
-						     dvdcss_error (handle));
+				/* take the first address for each extent of the file */
+				if (!file->specific.file.extents)
 					return FALSE;
+
+				for (extents = file->specific.file.extents; extents; extents = extents->next) {
+					BraseroVolFileExtent *extent;
+
+					extent = extents->data;
+					range->start = extent->block;
+					range->end = extent->block + BRASERO_SIZE_TO_SECTORS (extent->size, DVDCSS_BLOCK_SIZE);
+
+					g_queue_push_head (map, range);
+
+					if (dvdcss_seek (handle, range->start, DVDCSS_SEEK_KEY) != range->start) {
+						g_set_error (error,
+							     BRASERO_BURN_ERROR,
+							     BRASERO_BURN_ERROR_GENERAL,
+							     _("Error reading video DVD (%s)"),
+							     dvdcss_error (handle));
+						return FALSE;
+					}
 				}
 			}
 		}
@@ -286,15 +296,17 @@ brasero_dvdcss_write_image_thread (gpointer data)
 {
 	guchar buf [DVDCSS_BLOCK_SIZE * BRASERO_DVDCSS_I_BLOCKS];
 	BraseroScrambledSectorRange *range = NULL;
-	NautilusBurnDrive *drive = NULL;
+	BraseroMedium *medium = NULL;
 	BraseroVolFile *files = NULL;
 	dvdcss_handle *handle = NULL;
+	BraseroDrive *drive = NULL;
 	BraseroDvdcssPrivate *priv;
 	gint64 written_sectors = 0;
 	BraseroDvdcss *self = data;
 	BraseroTrack *track = NULL;
 	guint64 remaining_sectors;
 	FILE *output_fd = NULL;
+	BraseroVolSrc *vol;
 	gint64 volume_size;
 	GQueue *map = NULL;
 
@@ -310,16 +322,20 @@ brasero_dvdcss_write_image_thread (gpointer data)
 	/* get the contents of the DVD */
 	brasero_job_get_current_track (BRASERO_JOB (self), &track);
 	drive = brasero_track_get_drive_source (track);
-	files = brasero_volume_get_files (NCB_DRIVE_GET_DEVICE (drive),
+
+	vol = brasero_volume_source_open_file (brasero_drive_get_device (drive), &priv->error);
+	files = brasero_volume_get_files (vol,
 					  0,
 					  NULL,
 					  NULL,
 					  NULL,
 					  &priv->error);
+	brasero_volume_source_close (vol);
 	if (!files)
 		goto end;
 
-	NCB_MEDIA_GET_DATA_SIZE (drive, NULL, &volume_size);
+	medium = brasero_track_get_medium_source (track);
+	brasero_medium_get_data_size (medium, NULL, &volume_size);
 	if (volume_size == -1) {
 		priv->error = g_error_new (BRASERO_BURN_ERROR,
 					   BRASERO_BURN_ERROR_GENERAL,
@@ -328,7 +344,7 @@ brasero_dvdcss_write_image_thread (gpointer data)
 	}
 
 	/* create a handle/open DVD */
-	handle = dvdcss_open (NCB_DRIVE_GET_DEVICE (drive));
+	handle = dvdcss_open (brasero_drive_get_device (drive));
 	if (!handle) {
 		priv->error = g_error_new (BRASERO_BURN_ERROR,
 					   BRASERO_BURN_ERROR_GENERAL,
@@ -623,6 +639,7 @@ brasero_dvdcss_export_caps (BraseroPlugin *plugin, gchar **error)
 					 BRASERO_PLUGIN_IO_ACCEPT_PIPE,
 					 BRASERO_IMAGE_FORMAT_BIN);
 	input = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
+				       BRASERO_MEDIUM_DVD_DL|
 				       BRASERO_MEDIUM_ROM|
 				       BRASERO_MEDIUM_CLOSED|
 				       BRASERO_MEDIUM_HAS_DATA|
