@@ -107,12 +107,12 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const gchar *line)
 {
 	int perc_1, perc_2;
 
-	if (sscanf (line, " %2d.%1d%% done, estimate finish", &perc_1, &perc_2) == 2) {
+	if (sscanf (line, " %2d.%2d%% done, estimate finish", &perc_1, &perc_2) == 2) {
 		gdouble fraction;
 		BraseroBurnAction action;
 
 		fraction = (gdouble) ((gdouble) perc_1 +
-			   ((gdouble) perc_2 / (gdouble) 10.0)) /
+			   ((gdouble) perc_2 / (gdouble) 100.0)) /
 			   (gdouble) 100.0;
 
 		brasero_job_set_progress (BRASERO_JOB (process), fraction);
@@ -120,8 +120,10 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const gchar *line)
 		brasero_job_get_current_action (BRASERO_JOB (process), &action);
 		if (action == BRASERO_BURN_ACTION_BLANKING
 		&&  fraction >= 0.01) {
-			/* we nullified 1% (more than 65536) that's enough. A 
-			 * signal SIGTERM will be sent. */
+			/* we nullified 1% of the medium (more than 65536)
+			 * that's enough to make the filesystem unusable and
+			 * looking blank. A signal SIGTERM will be sent to stop
+			 * us. */
 			brasero_job_finished_session (BRASERO_JOB (process));
 			return BRASERO_BURN_OK;
 		}
@@ -231,6 +233,7 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 	BraseroBurnResult result;
 	BraseroTrackType input;
 	gchar *emptydir = NULL;
+	gchar *videodir = NULL;
 
 	g_ptr_array_add (argv, g_strdup ("-r"));
 
@@ -251,9 +254,16 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 	if (input.subtype.fs_type & BRASERO_IMAGE_FS_UDF)
 		g_ptr_array_add (argv, g_strdup ("-udf"));
 
-	if (input.subtype.fs_type & BRASERO_IMAGE_FS_VIDEO)
+	if (input.subtype.fs_type & BRASERO_IMAGE_FS_VIDEO) {
 		g_ptr_array_add (argv, g_strdup ("-dvd-video"));
 
+		result = brasero_job_get_tmp_dir (BRASERO_JOB (growisofs),
+						  &videodir,
+						  error);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	priv = BRASERO_GROWISOFS_PRIVATE (growisofs);
 	if (priv->use_utf8) {
 		g_ptr_array_add (argv, g_strdup ("-input-charset"));
@@ -269,8 +279,10 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 					   NULL,
 					   &grafts_path,
 					   error);
-	if (result != BRASERO_BURN_OK)
+	if (result != BRASERO_BURN_OK) {
+		g_free (videodir);
 		return result;
+	}
 
 	result = brasero_job_get_tmp_file (BRASERO_JOB (growisofs),
 					   NULL,
@@ -278,6 +290,7 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 					   error);
 	if (result != BRASERO_BURN_OK) {
 		g_free (grafts_path);
+		g_free (videodir);
 		return result;
 	}
 
@@ -285,6 +298,7 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 					  &emptydir,
 					  error);
 	if (result != BRASERO_BURN_OK) {
+		g_free (videodir);
 		g_free (grafts_path);
 		g_free (excluded_path);
 		return result;
@@ -294,10 +308,12 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 					       grafts_path,
 					       excluded_path,
 					       emptydir,
+					       videodir,
 					       error);
 	g_free (emptydir);
 
 	if (result != BRASERO_BURN_OK) {
+		g_free (videodir);
 		g_free (grafts_path);
 		g_free (excluded_path);
 		return result;
@@ -341,6 +357,11 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 		g_ptr_array_add (argv, g_strdup ("-print-size"));
 	}
 
+	if (videodir) {
+		g_ptr_array_add (argv, g_strdup ("-f"));
+		g_ptr_array_add (argv, videodir);
+	}
+
 	return BRASERO_BURN_OK;
 }
 
@@ -643,7 +664,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 			       "growisofs",
 			       _("growisofs burns DVDs"),
 			       "Philippe Rouquier",
-			       0);
+			       7);
 
 	/* First see if this plugin can be used, i.e. if growisofs is in
 	 * the path */
@@ -673,6 +694,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
 					BRASERO_MEDIUM_SEQUENTIAL|
 					BRASERO_MEDIUM_REWRITABLE|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_BLANK);
 
 	brasero_plugin_link_caps (plugin, output, input);
@@ -684,6 +706,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_MEDIUM_PLUS|
 					BRASERO_MEDIUM_RESTRICTED|
 					BRASERO_MEDIUM_REWRITABLE|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_BLANK|
 					BRASERO_MEDIUM_CLOSED|
 					BRASERO_MEDIUM_APPENDABLE|
@@ -701,6 +724,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_MEDIUM_JUMP|
 					BRASERO_MEDIUM_WRITABLE|
 					BRASERO_MEDIUM_REWRITABLE|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_BLANK|
 					BRASERO_MEDIUM_APPENDABLE|
 					BRASERO_MEDIUM_HAS_DATA);
@@ -736,6 +760,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_JUMP|
 				  BRASERO_MEDIUM_WRITABLE|
 				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_UNFORMATTED|
 				  BRASERO_MEDIUM_BLANK,
 				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
@@ -751,6 +776,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_JUMP|
 				  BRASERO_MEDIUM_WRITABLE|
 				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_UNFORMATTED|
 				  BRASERO_MEDIUM_BLANK,
 				  BRASERO_BURN_FLAG_DAO|
 				  BRASERO_BURN_FLAG_BURNPROOF|
@@ -782,6 +808,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_DVD|
 				  BRASERO_MEDIUM_RESTRICTED|
 				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_UNFORMATTED|
 				  BRASERO_MEDIUM_BLANK,
 				  BRASERO_BURN_FLAG_DAO|
 				  BRASERO_BURN_FLAG_MULTI|
@@ -865,6 +892,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_DVDRW_PLUS|
 				  BRASERO_MEDIUM_DVD_DL|
+				  BRASERO_MEDIUM_UNFORMATTED|
 				  BRASERO_MEDIUM_BLANK,
 				  BRASERO_BURN_FLAG_MULTI|
 				  BRASERO_BURN_FLAG_DAO|
@@ -908,6 +936,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_MEDIUM_APPENDABLE|
 					BRASERO_MEDIUM_CLOSED|
 					BRASERO_MEDIUM_HAS_DATA|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_BLANK);
 	brasero_plugin_blank_caps (plugin, output);
 	g_slist_free (output);
@@ -919,6 +948,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_MEDIUM_APPENDABLE|
 					BRASERO_MEDIUM_HAS_DATA|
 					BRASERO_MEDIUM_BLANK|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_CLOSED,
 					BRASERO_BURN_FLAG_NOGRACE|
 					BRASERO_BURN_FLAG_FAST_BLANK,
@@ -931,6 +961,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_MEDIUM_APPENDABLE|
 					BRASERO_MEDIUM_HAS_DATA|
 					BRASERO_MEDIUM_BLANK|
+					BRASERO_MEDIUM_UNFORMATTED|
 					BRASERO_MEDIUM_CLOSED,
 					BRASERO_BURN_FLAG_NOGRACE|
 					BRASERO_BURN_FLAG_FAST_BLANK,
